@@ -1,23 +1,18 @@
 <?php
 
 /**
- * This file is part of the Nette Framework (http://nette.org)
- * Copyright (c) 2004 David Grudl (http://davidgrudl.com)
+ * This file is part of the Nette Framework (https://nette.org)
+ * Copyright (c) 2004 David Grudl (https://davidgrudl.com)
  */
 
 namespace Nette\Application\Responses;
 
-use Nette,
-	Nette\Http;
+use Nette;
+use Nette\Http;
 
 
 /**
  * Redirects to new URI.
- *
- * @author     David Grudl
- *
- * @property-read string $url
- * @property-read int $code
  */
 class RedirectResponse extends Nette\Object implements Nette\Application\IResponse
 {
