@@ -1,34 +1,27 @@
 <?php
 
 /**
- * This file is part of the Nette Framework (http://nette.org)
- * Copyright (c) 2004 David Grudl (http://davidgrudl.com)
+ * This file is part of the Nette Framework (https://nette.org)
+ * Copyright (c) 2004 David Grudl (https://davidgrudl.com)
  */
 
 namespace Nette\Application\Routers;
 
-use Nette,
-	Nette\Application,
-	Nette\Utils\Strings;
+use Nette;
+use Nette\Application;
+use Nette\Utils\Strings;
 
 
 /**
  * The bidirectional route is responsible for mapping
  * HTTP request to a Request object for dispatch and vice-versa.
- *
- * @author     David Grudl
- *
- * @property-read string $mask
- * @property-read array $defaults
- * @property-read int $flags
- * @property-read string|FALSE $targetPresenter
  */
 class Route extends Nette\Object implements Application\IRouter
 {
 	const PRESENTER_KEY = 'presenter';
 	const MODULE_KEY = 'module';
 
-	/** flag */
+	/** @deprecated */
 	const CASE_SENSITIVE = 256;
 
 	/** @internal url type */
@@ -56,7 +49,6 @@ class Route extends Nette\Object implements Application\IRouter
 	public static $styles = array(
 		'#' => array( // default style for path parameters
 			self::PATTERN => '[^/]+',
-			self::FILTER_IN => 'rawurldecode',
 			self::FILTER_OUT => array(__CLASS__, 'param2path'),
 		),
 		'?#' => array( // default style for query parameters
@@ -93,6 +85,9 @@ class Route extends Nette\Object implements Application\IRouter
 	/** @var string  regular expression pattern */
 	private $re;
 
+	/** @var string[]  parameter aliases in regular expression */
+	private $aliases;
+
 	/** @var array of [value & fixity, filterIn, filterOut] */
 	private $metadata = array();
 
@@ -105,23 +100,29 @@ class Route extends Nette\Object implements Application\IRouter
 	/** @var int */
 	private $flags;
 
+	/** @var Nette\Http\Url */
+	private $lastRefUrl;
+
+	/** @var string */
+	private $lastBaseUrl;
+
 
 	/**
 	 * @param  string  URL mask, e.g. '<presenter>/<action>/<id \d{1,3}>'
-	 * @param  array|string   default values or metadata
+	 * @param  array|string|\Closure  default values or metadata or callback for NetteModule\MicroPresenter
 	 * @param  int     flags
 	 */
 	public function __construct($mask, $metadata = array(), $flags = 0)
 	{
 		if (is_string($metadata)) {
-			$a = strrpos($metadata, ':');
+			$a = strrpos($tmp = $metadata, ':');
 			if (!$a) {
 				throw new Nette\InvalidArgumentException("Second argument must be array or string in format Presenter:action, '$metadata' given.");
 			}
-			$metadata = array(
-				self::PRESENTER_KEY => substr($metadata, 0, $a),
-				'action' => $a === strlen($metadata) - 1 ? NULL : substr($metadata, $a + 1),
-			);
+			$metadata = array(self::PRESENTER_KEY => substr($tmp, 0, $a));
+			if ($a < strlen($tmp) - 1) {
+				$metadata['action'] = substr($tmp, $a + 1);
+			}
 		} elseif ($metadata instanceof \Closure || $metadata instanceof Nette\Callback) {
 			$metadata = array(
 				self::PRESENTER_KEY => 'Nette:Micro',
@@ -147,12 +148,13 @@ class Route extends Nette\Object implements Application\IRouter
 		$re = $this->re;
 
 		if ($this->type === self::HOST) {
-			$path = '//' . $url->getHost() . $url->getPath();
-			$host = array_reverse(explode('.', $url->getHost()));
+			$host = $url->getHost();
+			$path = '//' . $host . $url->getPath();
+			$host = ip2long($host) ? array($host) : array_reverse(explode('.', $host));
 			$re = strtr($re, array(
 				'/%basePath%/' => preg_quote($url->getBasePath(), '#'),
-				'%tld%' => $host[0],
-				'%domain%' => isset($host[1]) ? "$host[1]\\.$host[0]" : $host[0],
+				'%tld%' => preg_quote($host[0], '#'),
+				'%domain%' => preg_quote(isset($host[1]) ? "$host[1].$host[0]" : $host[0], '#'),
 			));
 
 		} elseif ($this->type === self::RELATIVE) {
@@ -167,7 +169,7 @@ class Route extends Nette\Object implements Application\IRouter
 		}
 
 		if ($path !== '') {
-			$path = rtrim($path, '/') . '/';
+			$path = rtrim(rawurldecode($path), '/') . '/';
 		}
 
 		if (!$matches = Strings::match($path, $re)) {
@@ -175,21 +177,18 @@ class Route extends Nette\Object implements Application\IRouter
 			return NULL;
 		}
 
-		// deletes numeric keys, restore '-' chars
+		// assigns matched values to parameters
 		$params = array();
 		foreach ($matches as $k => $v) {
 			if (is_string($k) && $v !== '') {
-				$params[str_replace('___', '-', $k)] = $v; // trick
+				$params[$this->aliases[$k]] = $v;
 			}
 		}
 
 
 		// 2) CONSTANT FIXITY
 		foreach ($this->metadata as $name => $meta) {
-			if (isset($params[$name])) {
-				//$params[$name] = $this->flags & self::CASE_SENSITIVE === 0 ? strtolower($params[$name]) : */$params[$name]; // strtolower damages UTF-8
-
-			} elseif (isset($meta['fixity']) && $meta['fixity'] !== self::OPTIONAL) {
+			if (!isset($params[$name]) && isset($meta['fixity']) && $meta['fixity'] !== self::OPTIONAL) {
 				$params[$name] = NULL; // cannot be overwriten in 3) and detected by isset() in 4)
 			}
 		}
@@ -239,16 +238,12 @@ class Route extends Nette\Object implements Application\IRouter
 		} elseif (!is_string($params[self::PRESENTER_KEY])) {
 			return NULL;
 		}
-		if (isset($this->metadata[self::MODULE_KEY])) {
-			if (!isset($params[self::MODULE_KEY])) {
-				throw new Nette\InvalidStateException('Missing module in route definition.');
-			}
-			$presenter = $params[self::MODULE_KEY] . ':' . $params[self::PRESENTER_KEY];
-			unset($params[self::MODULE_KEY], $params[self::PRESENTER_KEY]);
+		$presenter = $params[self::PRESENTER_KEY];
+		unset($params[self::PRESENTER_KEY]);
 
-		} else {
-			$presenter = $params[self::PRESENTER_KEY];
-			unset($params[self::PRESENTER_KEY]);
+		if (isset($this->metadata[self::MODULE_KEY])) {
+			$presenter = (isset($params[self::MODULE_KEY]) ? $params[self::MODULE_KEY] . ':' : '') . $presenter;
+			unset($params[self::MODULE_KEY]);
 		}
 
 		return new Application\Request(
@@ -287,13 +282,13 @@ class Route extends Nette\Object implements Application\IRouter
 
 		if (isset($metadata[self::MODULE_KEY])) { // try split into module and [submodule:]presenter parts
 			$module = $metadata[self::MODULE_KEY];
-			if (isset($module['fixity']) && strncasecmp($presenter, $module[self::VALUE] . ':', strlen($module[self::VALUE]) + 1) === 0) {
+			if (isset($module['fixity']) && strncmp($presenter, $module[self::VALUE] . ':', strlen($module[self::VALUE]) + 1) === 0) {
 				$a = strlen($module[self::VALUE]);
 			} else {
 				$a = strrpos($presenter, ':');
 			}
 			if ($a === FALSE) {
-				$params[self::MODULE_KEY] = '';
+				$params[self::MODULE_KEY] = isset($module[self::VALUE]) ? '' : NULL;
 			} else {
 				$params[self::MODULE_KEY] = substr($presenter, 0, $a);
 				$params[self::PRESENTER_KEY] = substr($presenter, $a + 1);
@@ -308,10 +303,11 @@ class Route extends Nette\Object implements Application\IRouter
 			if (isset($meta['fixity'])) {
 				if ($params[$name] === FALSE) {
 					$params[$name] = '0';
+				} elseif (is_scalar($params[$name])) {
+					$params[$name] = (string) $params[$name];
 				}
-				if (is_scalar($params[$name]) ? strcasecmp($params[$name], $meta[self::VALUE]) === 0
-					: $params[$name] === $meta[self::VALUE]
-				) { // remove default values; NULL values are retain
+
+				if ($params[$name] === $meta[self::VALUE]) { // remove default values; NULL values are retain
 					unset($params[$name]);
 					continue;
 
@@ -384,28 +380,30 @@ class Route extends Nette\Object implements Application\IRouter
 		} while (TRUE);
 
 
-		// absolutize path
-		if ($this->type === self::RELATIVE) {
-			$url = '//' . $refUrl->getAuthority() . $refUrl->getBasePath() . $url;
-
-		} elseif ($this->type === self::PATH) {
-			$url = '//' . $refUrl->getAuthority() . $url;
+		if ($this->type !== self::HOST) {
+			if ($this->lastRefUrl !== $refUrl) {
+				$scheme = ($this->flags & self::SECURED ? 'https://' : 'http://');
+				$basePath = ($this->type === self::RELATIVE ? $refUrl->getBasePath() : '');
+				$this->lastBaseUrl = $scheme . $refUrl->getAuthority() . $basePath;
+				$this->lastRefUrl = $refUrl;
+			}
+			$url = $this->lastBaseUrl . $url;
 
 		} else {
-			$host = array_reverse(explode('.', $refUrl->getHost()));
+			$host = $refUrl->getHost();
+			$host = ip2long($host) ? array($host) : array_reverse(explode('.', $host));
 			$url = strtr($url, array(
 				'/%basePath%/' => $refUrl->getBasePath(),
 				'%tld%' => $host[0],
 				'%domain%' => isset($host[1]) ? "$host[1].$host[0]" : $host[0],
 			));
+			$url = ($this->flags & self::SECURED ? 'https:' : 'http:') . $url;
 		}
 
-		if (strpos($url, '//', 2) !== FALSE) {
+		if (strpos($url, '//', 7) !== FALSE) {
 			return NULL;
 		}
 
-		$url = ($this->flags & self::SECURED ? 'https:' : 'http:') . $url;
-
 		// build query string
 		if ($this->xlat) {
 			$params = self::renameKeys($params, $this->xlat);
@@ -444,13 +442,24 @@ class Route extends Nette\Object implements Application\IRouter
 
 		foreach ($metadata as $name => $meta) {
 			if (!is_array($meta)) {
-				$metadata[$name] = array(self::VALUE => $meta, 'fixity' => self::CONSTANT);
+				$metadata[$name] = $meta = array(self::VALUE => $meta);
+			}
 
-			} elseif (array_key_exists(self::VALUE, $meta)) {
+			if (array_key_exists(self::VALUE, $meta)) {
+				if (is_scalar($meta[self::VALUE])) {
+					$metadata[$name][self::VALUE] = (string) $meta[self::VALUE];
+				}
 				$metadata[$name]['fixity'] = self::CONSTANT;
 			}
 		}
 
+		if (strpbrk($mask, '?<[') === FALSE) {
+			$this->re = '#' . preg_quote($mask, '#') . '/?\z#A';
+			$this->sequence = array($mask);
+			$this->metadata = $metadata;
+			return;
+		}
+
 		// PARSE MASK
 		// <parameter-name[=default] [pattern] [#class]> or [ or ] or ?...
 		$parts = Strings::split($mask, '/<([^>#= ]+)(=[^># ]*)? *([^>#]*)(#?[^>\[\]]*)>|(\[!?|\]|\s*\?.*)/');
@@ -503,6 +512,7 @@ class Route extends Nette\Object implements Application\IRouter
 		$re = '';
 		$sequence = array();
 		$autoOptional = TRUE;
+		$aliases = array();
 		do {
 			array_unshift($sequence, $parts[$i]);
 			$re = preg_quote($parts[$i], '#') . $re;
@@ -536,11 +546,6 @@ class Route extends Nette\Object implements Application\IRouter
 				continue;
 			}
 
-			// check name (limitation by regexp)
-			if (preg_match('#[^a-z0-9_-]#i', $name)) {
-				throw new Nette\InvalidArgumentException("Parameter name must be alphanumeric string due to limitations of PCRE, '$name' given.");
-			}
-
 			// pattern, condition & metadata
 			if ($class !== '') {
 				if (!isset(static::$styles[$class])) {
@@ -580,10 +585,11 @@ class Route extends Nette\Object implements Application\IRouter
 					$meta['defOut'] = $meta[self::VALUE];
 				}
 			}
-			$meta[self::PATTERN] = "#(?:$pattern)\\z#A" . ($this->flags & self::CASE_SENSITIVE ? '' : 'iu');
+			$meta[self::PATTERN] = "#(?:$pattern)\\z#A";
 
 			// include in expression
-			$re = '(?P<' . str_replace('-', '___', $name) . '>(?U)' . $pattern . ')' . $re; // str_replace is dirty trick to enable '-' in parameter name
+			$aliases['p' . $i] = $name;
+			$re = '(?P<p' . $i . '>(?U)' . $pattern . ')' . $re;
 			if ($brackets) { // is in brackets?
 				if (!isset($meta[self::VALUE])) {
 					$meta[self::VALUE] = $meta['defOut'] = NULL;
@@ -608,7 +614,8 @@ class Route extends Nette\Object implements Application\IRouter
 			throw new Nette\InvalidArgumentException("Missing closing ']' in mask '$mask'.");
 		}
 
-		$this->re = '#' . $re . '/?\z#A' . ($this->flags & self::CASE_SENSITIVE ? '' : 'iu');
+		$this->aliases = $aliases;
+		$this->re = '#' . $re . '/?\z#A';
 		$this->metadata = $metadata;
 		$this->sequence = $sequence;
 	}
@@ -656,12 +663,12 @@ class Route extends Nette\Object implements Application\IRouter
 	/**
 	 * Proprietary cache aim.
 	 * @internal
-	 * @return string|FALSE
+	 * @return string[]|NULL
 	 */
-	public function getTargetPresenter()
+	public function getTargetPresenters()
 	{
 		if ($this->flags & self::ONE_WAY) {
-			return FALSE;
+			return array();
 		}
 
 		$m = $this->metadata;
@@ -676,7 +683,7 @@ class Route extends Nette\Object implements Application\IRouter
 		}
 
 		if (isset($m[self::PRESENTER_KEY]['fixity']) && $m[self::PRESENTER_KEY]['fixity'] === self::CONSTANT) {
-			return $module . $m[self::PRESENTER_KEY][self::VALUE];
+			return array($module . $m[self::PRESENTER_KEY][self::VALUE]);
 		}
 		return NULL;
 	}
@@ -732,7 +739,6 @@ class Route extends Nette\Object implements Application\IRouter
 	 */
 	private static function path2action($s)
 	{
-		$s = strtolower($s);
 		$s = preg_replace('#-(?=[a-z])#', ' ', $s);
 		$s = lcfirst(ucwords($s));
 		$s = str_replace(' ', '', $s);
@@ -762,7 +768,6 @@ class Route extends Nette\Object implements Application\IRouter
 	 */
 	private static function path2presenter($s)
 	{
-		$s = strtolower($s);
 		$s = preg_replace('#([.-])(?=[a-z])#', '$1 ', $s);
 		$s = ucwords($s);
 		$s = str_replace('. ', ':', $s);
@@ -786,13 +791,11 @@ class Route extends Nette\Object implements Application\IRouter
 
 
 	/**
-	 * Creates new style.
-	 * @param  string  style name (#style, urlParameter, ?queryParameter)
-	 * @param  string  optional parent style name
-	 * @return void
+	 * @deprecated
 	 */
 	public static function addStyle($style, $parent = '#')
 	{
+		trigger_error(__METHOD__ . '() is deprecated.', E_USER_DEPRECATED);
 		if (isset(static::$styles[$style])) {
 			throw new Nette\InvalidArgumentException("Style '$style' already exists.");
 		}
@@ -810,14 +813,11 @@ class Route extends Nette\Object implements Application\IRouter
 
 
 	/**
-	 * Changes style property value.
-	 * @param  string  style name (#style, urlParameter, ?queryParameter)
-	 * @param  string  property name (Route::PATTERN, Route::FILTER_IN, Route::FILTER_OUT, Route::FILTER_TABLE)
-	 * @param  mixed   property value
-	 * @return void
+	 * @deprecated
 	 */
 	public static function setStyleProperty($style, $key, $value)
 	{
+		trigger_error(__METHOD__ . '() is deprecated.', E_USER_DEPRECATED);
 		if (!isset(static::$styles[$style])) {
 			throw new Nette\InvalidArgumentException("Style '$style' doesn't exist.");
 		}
