@@ -1,43 +1,37 @@
 <?php
 
 /**
- * This file is part of the Nette Framework (http://nette.org)
- * Copyright (c) 2004 David Grudl (http://davidgrudl.com)
+ * This file is part of the Nette Framework (https://nette.org)
+ * Copyright (c) 2004 David Grudl (https://davidgrudl.com)
  */
 
 namespace Nette\Application\UI;
 
-use Nette,
-	Nette\Application,
-	Nette\Application\Responses,
-	Nette\Http,
-	Nette\Reflection;
+use Nette;
+use Nette\Application;
+use Nette\Application\Responses;
+use Nette\Http;
 
 
 /**
  * Presenter component represents a webpage instance. It converts Request to IResponse.
  *
- * @author     David Grudl
- *
  * @property-read Nette\Application\Request $request
- * @property-read array|NULL $signal
  * @property-read string $action
  * @property      string $view
  * @property      string $layout
  * @property-read \stdClass $payload
- * @property-read bool $ajax
- * @property-read Nette\Application\Request $lastCreatedRequest
- * @property-read Nette\Http\SessionSection $flashSession
- * @property-read \SystemContainer|Nette\DI\Container $context
+ * @property-read Nette\DI\Container $context
  * @property-read Nette\Http\Session $session
  * @property-read Nette\Security\User $user
  */
 abstract class Presenter extends Control implements Application\IPresenter
 {
 	/** bad link handling {@link Presenter::$invalidLinkMode} */
-	const INVALID_LINK_SILENT = 1,
-		INVALID_LINK_WARNING = 2,
-		INVALID_LINK_EXCEPTION = 3;
+	const INVALID_LINK_SILENT = 0,
+		INVALID_LINK_WARNING = 1,
+		INVALID_LINK_EXCEPTION = 2,
+		INVALID_LINK_TEXTUAL = 4;
 
 	/** @internal special parameter key */
 	const SIGNAL_KEY = 'do',
@@ -48,7 +42,7 @@ abstract class Presenter extends Control implements Application\IPresenter
 	/** @var int */
 	public $invalidLinkMode;
 
-	/** @var array of function(Presenter $sender, IResponse $response = NULL); Occurs when the presenter is shutting down */
+	/** @var callable[]  function (Presenter $sender, IResponse $response = NULL); Occurs when the presenter is shutting down */
 	public $onShutdown;
 
 	/** @var Nette\Application\Request */
@@ -102,15 +96,9 @@ abstract class Presenter extends Control implements Application\IPresenter
 	/** @var array */
 	private $lastCreatedRequestFlag;
 
-	/** @var \SystemContainer|Nette\DI\Container */
+	/** @var Nette\DI\Container */
 	private $context;
 
-	/** @var Nette\Application\Application */
-	private $application;
-
-	/** @var Nette\Http\Context */
-	private $httpContext;
-
 	/** @var Nette\Http\IRequest */
 	private $httpRequest;
 
@@ -120,12 +108,22 @@ abstract class Presenter extends Control implements Application\IPresenter
 	/** @var Nette\Http\Session */
 	private $session;
 
+	/** @var Nette\Application\IPresenterFactory */
+	private $presenterFactory;
+
+	/** @var Nette\Application\IRouter */
+	private $router;
+
 	/** @var Nette\Security\User */
 	private $user;
 
+	/** @var ITemplateFactory */
+	private $templateFactory;
+
 
 	public function __construct()
 	{
+		$this->payload = new \stdClass;
 	}
 
 
@@ -169,7 +167,7 @@ abstract class Presenter extends Control implements Application\IPresenter
 		try {
 			// STARTUP
 			$this->request = $request;
-			$this->payload = new \stdClass;
+			$this->payload = $this->payload ?: new \stdClass;
 			$this->setParent($this->getParent(), $request->getPresenterName());
 
 			if (!$this->httpResponse->isSent()) {
@@ -215,21 +213,26 @@ abstract class Presenter extends Control implements Application\IPresenter
 			}
 
 			// finish template rendering
-			$this->sendTemplate();
+			if ($this->getTemplate()) {
+				$this->sendTemplate();
+			}
 
 		} catch (Application\AbortException $e) {
 			// continue with shutting down
-			if ($this->isAjax()) try {
-				$hasPayload = (array) $this->payload; unset($hasPayload['state']);
-				if ($this->response instanceof Responses\TextResponse && $this->isControlInvalid()) {
-					$this->snippetMode = TRUE;
-					$this->response->send($this->httpRequest, $this->httpResponse);
-					$this->sendPayload();
-
-				} elseif (!$this->response && $hasPayload) { // back compatibility for use terminate() instead of sendPayload()
-					$this->sendPayload();
+			if ($this->isAjax()) {
+				try {
+					$hasPayload = (array) $this->payload;
+					unset($hasPayload['state']);
+					if ($this->response instanceof Responses\TextResponse && $this->isControlInvalid()) {
+						$this->snippetMode = TRUE;
+						$this->response->send($this->httpRequest, $this->httpResponse);
+						$this->sendPayload();
+					} elseif (!$this->response && $hasPayload) { // back compatibility for use terminate() instead of sendPayload()
+						$this->sendPayload();
+					}
+				} catch (Application\AbortException $e) {
 				}
-			} catch (Application\AbortException $e) { }
+			}
 
 			if ($this->hasFlashSession()) {
 				$this->getFlashSession()->setExpiration($this->response instanceof Responses\RedirectResponse ? '+ 30 seconds' : '+ 3 seconds');
@@ -286,8 +289,8 @@ abstract class Presenter extends Control implements Application\IPresenter
 	 */
 	public function checkRequirements($element)
 	{
-		$user = (array) $element->getAnnotation('User');
-		if (in_array('loggedIn', $user, TRUE) && !$this->user->isLoggedIn()) {
+		$user = (array) PresenterComponentReflection::parseAnnotation($element, 'User');
+		if (in_array('loggedIn', $user, TRUE) && !$this->getUser()->isLoggedIn()) {
 			throw new Application\ForbiddenRequestException;
 		}
 	}
@@ -308,7 +311,8 @@ abstract class Presenter extends Control implements Application\IPresenter
 
 		try {
 			$component = $this->signalReceiver === '' ? $this : $this->getComponent($this->signalReceiver, FALSE);
-		} catch (Nette\InvalidArgumentException $e) {}
+		} catch (Nette\InvalidArgumentException $e) {
+		}
 
 		if (isset($e) || $component === NULL) {
 			throw new BadSignalException("The signal receiver component '$this->signalReceiver' is not found.", NULL, isset($e) ? $e : NULL);
@@ -442,11 +446,7 @@ abstract class Presenter extends Control implements Application\IPresenter
 	public function sendTemplate()
 	{
 		$template = $this->getTemplate();
-		if (!$template) {
-			return;
-		}
-
-		if ($template instanceof Nette\Templating\IFileTemplate && !$template->getFile()) { // content template
+		if (!$template->getFile()) {
 			$files = $this->formatTemplateFiles();
 			foreach ($files as $file) {
 				if (is_file($file)) {
@@ -505,12 +505,9 @@ abstract class Presenter extends Control implements Application\IPresenter
 		$list = array(
 			"$dir/templates/$presenter/@$layout.latte",
 			"$dir/templates/$presenter.@$layout.latte",
-			"$dir/templates/$presenter/@$layout.phtml",
-			"$dir/templates/$presenter.@$layout.phtml",
 		);
 		do {
 			$list[] = "$dir/templates/@$layout.latte";
-			$list[] = "$dir/templates/@$layout.phtml";
 			$dir = dirname($dir);
 		} while ($dir && ($name = substr($name, 0, strrpos($name, ':'))));
 		return $list;
@@ -530,8 +527,6 @@ abstract class Presenter extends Control implements Application\IPresenter
 		return array(
 			"$dir/templates/$presenter/$this->view.latte",
 			"$dir/templates/$presenter.$this->view.latte",
-			"$dir/templates/$presenter/$this->view.phtml",
-			"$dir/templates/$presenter.$this->view.phtml",
 		);
 	}
 
@@ -558,6 +553,15 @@ abstract class Presenter extends Control implements Application\IPresenter
 	}
 
 
+	/**
+	 * @return ITemplate
+	 */
+	protected function createTemplate()
+	{
+		return $this->getTemplateFactory()->createTemplate($this);
+	}
+
+
 	/********************* partial AJAX rendering ****************d*g**/
 
 
@@ -596,7 +600,7 @@ abstract class Presenter extends Control implements Application\IPresenter
 
 	/**
 	 * Sends JSON data to the output.
-	 * @param  mixed $data
+	 * @param  mixed
 	 * @return void
 	 * @throws Nette\Application\AbortException
 	 */
@@ -628,10 +632,6 @@ abstract class Presenter extends Control implements Application\IPresenter
 	 */
 	public function terminate()
 	{
-		if (func_num_args() !== 0) {
-			trigger_error(__METHOD__ . ' is not intended to send a Application\Response; use sendResponse() instead.', E_USER_WARNING);
-			$this->sendResponse(func_get_arg(0));
-		}
 		throw new Application\AbortException();
 	}
 
@@ -733,7 +733,8 @@ abstract class Presenter extends Control implements Application\IPresenter
 		if (!$this->isAjax() && ($this->request->isMethod('get') || $this->request->isMethod('head'))) {
 			try {
 				$url = $this->createRequest($this, $this->action, $this->getGlobalState() + $this->request->getParameters(), 'redirectX');
-			} catch (InvalidLinkException $e) {}
+			} catch (InvalidLinkException $e) {
+			}
 			if (isset($url) && !$this->httpRequest->getUrl()->isEqual($url)) {
 				$this->sendResponse(new Responses\RedirectResponse($url, Http\IResponse::S301_MOVED_PERMANENTLY));
 			}
@@ -743,7 +744,7 @@ abstract class Presenter extends Control implements Application\IPresenter
 
 	/**
 	 * Attempts to cache the sent entity by its last modification date.
-	 * @param  string|int|DateTime  last modified time
+	 * @param  string|int|\DateTime  last modified time
 	 * @param  string strong entity tag validator
 	 * @param  mixed  optional expiration time
 	 * @return void
@@ -754,8 +755,8 @@ abstract class Presenter extends Control implements Application\IPresenter
 		if ($expire !== NULL) {
 			$this->httpResponse->setExpiration($expire);
 		}
-
-		if (!$this->httpContext->isModified($lastModified, $etag)) {
+		$helper = new Http\Context($this->httpRequest, $this->httpResponse);
+		if (!$helper->isModified($lastModified, $etag)) {
 			$this->terminate();
 		}
 	}
@@ -764,7 +765,7 @@ abstract class Presenter extends Control implements Application\IPresenter
 	/**
 	 * Request/URL factory.
 	 * @param  PresenterComponent  base
-	 * @param  string   destination in format "[[module:]presenter:]action" or "signal!" or "this"
+	 * @param  string   destination in format "[//] [[[module:]presenter:]action | signal! | this] [#fragment]"
 	 * @param  array    array of arguments
 	 * @param  string   forward|redirect|link
 	 * @return string   URL
@@ -775,15 +776,6 @@ abstract class Presenter extends Control implements Application\IPresenter
 	{
 		// note: createRequest supposes that saveState(), run() & tryCall() behaviour is final
 
-		// cached services for better performance
-		static $presenterFactory, $router, $refUrl;
-		if ($presenterFactory === NULL) {
-			$presenterFactory = $this->application->getPresenterFactory();
-			$router = $this->application->getRouter();
-			$refUrl = new Http\Url($this->httpRequest->getUrl());
-			$refUrl->setPath($this->httpRequest->getUrl()->getScriptPath());
-		}
-
 		$this->lastCreatedRequest = $this->lastCreatedRequestFlag = NULL;
 
 		// PARSE DESTINATION
@@ -813,7 +805,7 @@ abstract class Presenter extends Control implements Application\IPresenter
 		}
 
 		// 4) signal or empty
-		if (!$component instanceof Presenter || substr($destination, -1) === '!') {
+		if (!$component instanceof self || substr($destination, -1) === '!') {
 			$signal = rtrim($destination, '!');
 			$a = strrpos($signal, ':');
 			if ($a !== FALSE) {
@@ -821,13 +813,13 @@ abstract class Presenter extends Control implements Application\IPresenter
 				$signal = (string) substr($signal, $a + 1);
 			}
 			if ($signal == NULL) {  // intentionally ==
-				throw new InvalidLinkException("Signal must be non-empty string.");
+				throw new InvalidLinkException('Signal must be non-empty string.');
 			}
 			$destination = 'this';
 		}
 
 		if ($destination == NULL) {  // intentionally ==
-			throw new InvalidLinkException("Destination must be non-empty string.");
+			throw new InvalidLinkException('Destination must be non-empty string.');
 		}
 
 		// 5) presenter: action
@@ -855,8 +847,11 @@ abstract class Presenter extends Control implements Application\IPresenter
 					$presenter = substr($presenter, 0, $b + 1) . substr($destination, 0, $a);
 				}
 			}
+			if (!$this->presenterFactory) {
+				throw new Nette\InvalidStateException('Unable to create link to other presenter, service PresenterFactory has not been set.');
+			}
 			try {
-				$presenterClass = $presenterFactory->getPresenterClass($presenter);
+				$presenterClass = $this->presenterFactory->getPresenterClass($presenter);
 			} catch (Application\InvalidPresenterException $e) {
 				throw new InvalidLinkException($e->getMessage(), NULL, $e);
 			}
@@ -875,11 +870,10 @@ abstract class Presenter extends Control implements Application\IPresenter
 				// counterpart of signalReceived() & tryCall()
 				$method = $component->formatSignalMethod($signal);
 				if (!$reflection->hasCallableMethod($method)) {
-					throw new InvalidLinkException("Unknown signal '$signal', missing handler {$reflection->name}::$method()");
-				}
-				if ($args) { // convert indexed parameters to named
-					self::argsToParams(get_class($component), $method, $args);
+					throw new InvalidLinkException("Unknown signal '$signal', missing handler {$reflection->getName()}::$method()");
 				}
+				// convert indexed parameters to named
+				self::argsToParams(get_class($component), $method, $args, array(), $mode === 'test');
 			}
 
 			// counterpart of IStatePersistent
@@ -905,28 +899,23 @@ abstract class Presenter extends Control implements Application\IPresenter
 			$current = ($action === '*' || strcasecmp($action, $this->action) === 0) && $presenterClass === get_class($this);
 
 			$reflection = new PresenterComponentReflection($presenterClass);
-			if ($args || $destination === 'this') {
-				// counterpart of run() & tryCall()
-				$method = $presenterClass::formatActionMethod($action);
+
+			// counterpart of run() & tryCall()
+			$method = $presenterClass::formatActionMethod($action);
+			if (!$reflection->hasCallableMethod($method)) {
+				$method = $presenterClass::formatRenderMethod($action);
 				if (!$reflection->hasCallableMethod($method)) {
-					$method = $presenterClass::formatRenderMethod($action);
-					if (!$reflection->hasCallableMethod($method)) {
-						$method = NULL;
-					}
+					$method = NULL;
 				}
+			}
 
-				// convert indexed parameters to named
-				if ($method === NULL) {
-					if (array_key_exists(0, $args)) {
-						throw new InvalidLinkException("Unable to pass parameters to action '$presenter:$action', missing corresponding method.");
-					}
-
-				} elseif ($destination === 'this') {
-					self::argsToParams($presenterClass, $method, $args, $this->params);
-
-				} else {
-					self::argsToParams($presenterClass, $method, $args);
+			// convert indexed parameters to named
+			if ($method === NULL) {
+				if (array_key_exists(0, $args)) {
+					throw new InvalidLinkException("Unable to pass parameters to action '$presenter:$action', missing corresponding method.");
 				}
+			} else {
+				self::argsToParams($presenterClass, $method, $args, $destination === 'this' ? $this->params : array(), $mode === 'test');
 			}
 
 			// counterpart of IStatePersistent
@@ -977,7 +966,15 @@ abstract class Presenter extends Control implements Application\IPresenter
 		}
 
 		// CONSTRUCT URL
-		$url = $router->constructUrl($this->lastCreatedRequest, $refUrl);
+		static $refUrl;
+		if ($refUrl === NULL) {
+			$refUrl = new Http\Url($this->httpRequest->getUrl());
+			$refUrl->setPath($this->httpRequest->getUrl()->getScriptPath());
+		}
+		if (!$this->router) {
+			throw new Nette\InvalidStateException('Unable to generate URL, service Router has not been set.');
+		}
+		$url = $this->router->constructUrl($this->lastCreatedRequest, $refUrl);
 		if ($url === NULL) {
 			unset($args[self::ACTION_KEY]);
 			$params = urldecode(http_build_query($args, NULL, ', '));
@@ -1002,15 +999,19 @@ abstract class Presenter extends Control implements Application\IPresenter
 	 * @param  string  method name
 	 * @param  array   arguments
 	 * @param  array   supplemental arguments
+	 * @param  bool    prevents 'Missing parameter' exception
 	 * @return void
 	 * @throws InvalidLinkException
+	 * @internal
 	 */
-	private static function argsToParams($class, $method, & $args, $supplemental = array())
+	public static function argsToParams($class, $method, & $args, $supplemental = array(), $ignoreMissing = FALSE)
 	{
 		$i = 0;
 		$rm = new \ReflectionMethod($class, $method);
 		foreach ($rm->getParameters() as $param) {
+			list($type, $isClass) = PresenterComponentReflection::getParameterType($param);
 			$name = $param->getName();
+
 			if (array_key_exists($i, $args)) {
 				$args[$name] = $args[$i];
 				unset($args[$i]);
@@ -1021,29 +1022,33 @@ abstract class Presenter extends Control implements Application\IPresenter
 
 			} elseif (array_key_exists($name, $supplemental)) {
 				$args[$name] = $supplemental[$name];
-
-			} else {
-				continue;
 			}
 
-			if ($args[$name] === NULL) {
-				continue;
+			if (!isset($args[$name])) {
+				if ($param->isDefaultValueAvailable() || $type === 'NULL' || $type === 'array' || $isClass || $ignoreMissing) {
+					continue;
+				}
+				throw new InvalidLinkException("Missing parameter \$$name required by $class::{$rm->getName()}()");
 			}
 
-			$def = $param->isDefaultValueAvailable() && $param->isOptional() ? $param->getDefaultValue() : NULL; // see PHP bug #62988
-			$type = $param->isArray() ? 'array' : gettype($def);
-			if (!PresenterComponentReflection::convertType($args[$name], $type)) {
-				throw new InvalidLinkException("Invalid value for parameter '$name' in method $class::$method(), expected " . ($type === 'NULL' ? 'scalar' : $type) . ".");
+			if (!PresenterComponentReflection::convertType($args[$name], $type, $isClass)) {
+				throw new InvalidLinkException(sprintf(
+					'Argument $%s passed to %s() must be %s, %s given.',
+					$name,
+					$rm->getDeclaringClass()->getName() . '::' . $rm->getName(),
+					$type === 'NULL' ? 'scalar' : $type,
+					is_object($args[$name]) ? get_class($args[$name]) : gettype($args[$name])
+				));
 			}
 
+			$def = $param->isDefaultValueAvailable() ? $param->getDefaultValue() : NULL;
 			if ($args[$name] === $def || ($def === NULL && is_scalar($args[$name]) && (string) $args[$name] === '')) {
 				$args[$name] = NULL; // value transmit is unnecessary
 			}
 		}
 
 		if (array_key_exists($i, $args)) {
-			$method = $rm->getName();
-			throw new InvalidLinkException("Passed more parameters than method $class::$method() expects.");
+			throw new InvalidLinkException("Passed more parameters than method $class::{$rm->getName()}() expects.");
 		}
 	}
 
@@ -1055,15 +1060,14 @@ abstract class Presenter extends Control implements Application\IPresenter
 	 */
 	protected function handleInvalidLink(InvalidLinkException $e)
 	{
-		if ($this->invalidLinkMode === self::INVALID_LINK_SILENT) {
-			return '#';
-
-		} elseif ($this->invalidLinkMode === self::INVALID_LINK_WARNING) {
-			return 'error: ' . $e->getMessage();
-
-		} else { // self::INVALID_LINK_EXCEPTION
+		if ($this->invalidLinkMode & self::INVALID_LINK_EXCEPTION) {
 			throw $e;
+		} elseif ($this->invalidLinkMode & self::INVALID_LINK_WARNING) {
+			trigger_error('Invalid link: ' . $e->getMessage(), E_USER_WARNING);
 		}
+		return $this->invalidLinkMode & self::INVALID_LINK_TEXTUAL
+			? '#error: ' . $e->getMessage()
+			: '#';
 	}
 
 
@@ -1077,12 +1081,12 @@ abstract class Presenter extends Control implements Application\IPresenter
 	 */
 	public function storeRequest($expiration = '+ 10 minutes')
 	{
-		$session = $this->session->getSection('Nette.Application/requests');
+		$session = $this->getSession('Nette.Application/requests');
 		do {
-			$key = Nette\Utils\Strings::random(5);
+			$key = Nette\Utils\Random::generate(5);
 		} while (isset($session[$key]));
 
-		$session[$key] = array($this->user->getId(), $this->request);
+		$session[$key] = array($this->getUser()->getId(), $this->request);
 		$session->setExpiration($expiration, $key);
 		return $key;
 	}
@@ -1095,8 +1099,8 @@ abstract class Presenter extends Control implements Application\IPresenter
 	 */
 	public function restoreRequest($key)
 	{
-		$session = $this->session->getSection('Nette.Application/requests');
-		if (!isset($session[$key]) || ($session[$key][0] !== NULL && $session[$key][0] !== $this->user->getId())) {
+		$session = $this->getSession('Nette.Application/requests');
+		if (!isset($session[$key]) || ($session[$key][0] !== NULL && $session[$key][0] !== $this->getUser()->getId())) {
 			return;
 		}
 		$request = clone $session[$key][1];
@@ -1119,7 +1123,7 @@ abstract class Presenter extends Control implements Application\IPresenter
 	 */
 	public static function getPersistentComponents()
 	{
-		return (array) Reflection\ClassType::from(get_called_class())->getAnnotation('persistent');
+		return (array) PresenterComponentReflection::parseAnnotation(new \ReflectionClass(get_called_class()), 'persistent');
 	}
 
 
@@ -1127,7 +1131,7 @@ abstract class Presenter extends Control implements Application\IPresenter
 	 * Saves state information for all subcomponents to $this->globalState.
 	 * @return array
 	 */
-	private function getGlobalState($forClass = NULL)
+	protected function getGlobalState($forClass = NULL)
 	{
 		$sinces = & $this->globalStateSinces;
 
@@ -1217,6 +1221,9 @@ abstract class Presenter extends Control implements Application\IPresenter
 		if ($this->isAjax()) {
 			$params += $this->request->getPost();
 		}
+		if (($tmp = $this->request->getPost(self::SIGNAL_KEY)) !== NULL) {
+			$params[self::SIGNAL_KEY] = $tmp;
+		}
 
 		foreach ($params as $key => $value) {
 			if (!preg_match('#^((?:[a-z0-9_]+-)*)((?!\d+\z)[a-z0-9_]+)\z#i', $key, $matches)) {
@@ -1284,7 +1291,7 @@ abstract class Presenter extends Control implements Application\IPresenter
 	public function hasFlashSession()
 	{
 		return !empty($this->params[self::FLASH_KEY])
-			&& $this->session->hasSection('Nette.Application.Flash/' . $this->params[self::FLASH_KEY]);
+			&& $this->getSession()->hasSection('Nette.Application.Flash/' . $this->params[self::FLASH_KEY]);
 	}
 
 
@@ -1295,52 +1302,47 @@ abstract class Presenter extends Control implements Application\IPresenter
 	public function getFlashSession()
 	{
 		if (empty($this->params[self::FLASH_KEY])) {
-			$this->params[self::FLASH_KEY] = Nette\Utils\Strings::random(4);
+			$this->params[self::FLASH_KEY] = Nette\Utils\Random::generate(4);
 		}
-		return $this->session->getSection('Nette.Application.Flash/' . $this->params[self::FLASH_KEY]);
+		return $this->getSession('Nette.Application.Flash/' . $this->params[self::FLASH_KEY]);
 	}
 
 
 	/********************* services ****************d*g**/
 
 
-	public function injectPrimary(Nette\DI\Container $context, Application\Application $application, Http\Context $httpContext, Http\IRequest $httpRequest, Http\IResponse $httpResponse, Http\Session $session, Nette\Security\User $user)
+	public function injectPrimary(Nette\DI\Container $context = NULL, Application\IPresenterFactory $presenterFactory = NULL, Application\IRouter $router = NULL,
+		Http\IRequest $httpRequest, Http\IResponse $httpResponse, Http\Session $session = NULL, Nette\Security\User $user = NULL, ITemplateFactory $templateFactory = NULL)
 	{
-		if ($this->application !== NULL) {
-			throw new Nette\InvalidStateException("Method " . __METHOD__ . " is intended for initialization and should not be called more than once.");
+		if ($this->presenterFactory !== NULL) {
+			throw new Nette\InvalidStateException('Method ' . __METHOD__ . ' is intended for initialization and should not be called more than once.');
 		}
 
 		$this->context = $context;
-		$this->application = $application;
-		$this->httpContext = $httpContext;
+		$this->presenterFactory = $presenterFactory;
+		$this->router = $router;
 		$this->httpRequest = $httpRequest;
 		$this->httpResponse = $httpResponse;
 		$this->session = $session;
 		$this->user = $user;
+		$this->templateFactory = $templateFactory;
 	}
 
 
 	/**
 	 * Gets the context.
-	 * @return \SystemContainer|Nette\DI\Container
+	 * @return Nette\DI\Container
 	 * @deprecated
 	 */
 	public function getContext()
 	{
+		if (!$this->context) {
+			throw new Nette\InvalidStateException('Context has not been set.');
+		}
 		return $this->context;
 	}
 
 
-	/**
-	 * @deprecated
-	 */
-	public function getService($name)
-	{
-		trigger_error(__METHOD__ . '() is deprecated; use dependency injection instead.', E_USER_DEPRECATED);
-		return $this->context->getService($name);
-	}
-
-
 	/**
 	 * @return Nette\Http\IRequest
 	 */
@@ -1359,34 +1361,16 @@ abstract class Presenter extends Control implements Application\IPresenter
 	}
 
 
-	/**
-	 * @deprecated
-	 */
-	protected function getHttpContext()
-	{
-		trigger_error(__METHOD__ . '() is deprecated; use dependency injection instead.', E_USER_DEPRECATED);
-		return $this->httpContext;
-	}
-
-
-	/**
-	 * @deprecated
-	 */
-	public function getApplication()
-	{
-		trigger_error(__METHOD__ . '() is deprecated; use dependency injection instead.', E_USER_DEPRECATED);
-		return $this->application;
-	}
-
-
 	/**
 	 * @param  string
 	 * @return Nette\Http\Session|Nette\Http\SessionSection
 	 */
 	public function getSession($namespace = NULL)
 	{
-		$handler = $this->session;
-		return $namespace === NULL ? $handler : $handler->getSection($namespace);
+		if (!$this->session) {
+			throw new Nette\InvalidStateException('Service Session has not been set.');
+		}
+		return $namespace === NULL ? $this->session : $this->session->getSection($namespace);
 	}
 
 
@@ -1395,7 +1379,22 @@ abstract class Presenter extends Control implements Application\IPresenter
 	 */
 	public function getUser()
 	{
+		if (!$this->user) {
+			throw new Nette\InvalidStateException('Service User has not been set.');
+		}
 		return $this->user;
 	}
 
+
+	/**
+	 * @return ITemplateFactory
+	 */
+	public function getTemplateFactory()
+	{
+		if (!$this->templateFactory) {
+			throw new Nette\InvalidStateException('Service TemplateFactory has not been set.');
+		}
+		return $this->templateFactory;
+	}
+
 }
