@@ -1,8 +1,8 @@
 <?php
 
 /**
- * This file is part of the Nette Framework (http://nette.org)
- * Copyright (c) 2004 David Grudl (http://davidgrudl.com)
+ * This file is part of the Nette Framework (https://nette.org)
+ * Copyright (c) 2004 David Grudl (https://davidgrudl.com)
  */
 
 namespace Nette\DI;
@@ -12,18 +12,18 @@ use Nette;
 
 /**
  * The dependency injection container default implementation.
- *
- * @author     David Grudl
  */
 class Container extends Nette\Object
 {
 	const TAGS = 'tags';
 	const TYPES = 'types';
+	const SERVICES = 'services';
+	const ALIASES = 'aliases';
 
 	/** @var array  user parameters */
 	/*private*/public $parameters = array();
 
-	/** @var array  storage for shared objects */
+	/** @var object[]  storage for shared objects */
 	private $registry = array();
 
 	/** @var array[] */
@@ -56,22 +56,19 @@ class Container extends Nette\Object
 	 */
 	public function addService($name, $service)
 	{
-		if (func_num_args() > 2) {
-			throw new Nette\DeprecatedException('Parameter $meta has been removed.');
-
-		} elseif (!is_string($name) || !$name) {
+		if (!is_string($name) || !$name) {
 			throw new Nette\InvalidArgumentException(sprintf('Service name must be a non-empty string, %s given.', gettype($name)));
 
-		} elseif (isset($this->registry[$name])) {
+		}
+		$name = isset($this->meta[self::ALIASES][$name]) ? $this->meta[self::ALIASES][$name] : $name;
+		if (isset($this->registry[$name])) {
 			throw new Nette\InvalidStateException("Service '$name' already exists.");
 
-		} elseif (is_string($service) || is_array($service) || $service instanceof \Closure || $service instanceof Nette\Callback) {
-			trigger_error(sprintf('Passing factories to %s() is deprecated; pass the object itself.', __METHOD__), E_USER_DEPRECATED);
-			$service = is_string($service) && !preg_match('#\x00|:#', $service) ? new $service : call_user_func($service, $this);
-		}
+		} elseif (!is_object($service)) {
+			throw new Nette\InvalidArgumentException(sprintf("Service '%s' must be a object, %s given.", $name, gettype($service)));
 
-		if (!is_object($service)) {
-			throw new Nette\InvalidArgumentException(sprintf('Service must be a object, %s given.', gettype($service)));
+		} elseif (isset($this->meta[self::SERVICES][$name]) && !$service instanceof $this->meta[self::SERVICES][$name]) {
+			throw new Nette\InvalidArgumentException(sprintf("Service '%s' must be instance of %s, %s given.", $name, $this->meta[self::SERVICES][$name], get_class($service)));
 		}
 
 		$this->registry[$name] = $service;
@@ -86,6 +83,7 @@ class Container extends Nette\Object
 	 */
 	public function removeService($name)
 	{
+		$name = isset($this->meta[self::ALIASES][$name]) ? $this->meta[self::ALIASES][$name] : $name;
 		unset($this->registry[$name]);
 	}
 
@@ -99,12 +97,35 @@ class Container extends Nette\Object
 	public function getService($name)
 	{
 		if (!isset($this->registry[$name])) {
+			if (isset($this->meta[self::ALIASES][$name])) {
+				return $this->getService($this->meta[self::ALIASES][$name]);
+			}
 			$this->registry[$name] = $this->createService($name);
 		}
 		return $this->registry[$name];
 	}
 
 
+	/**
+	 * Gets the service type by name.
+	 * @param  string
+	 * @return string
+	 * @throws MissingServiceException
+	 */
+	public function getServiceType($name)
+	{
+		if (isset($this->meta[self::ALIASES][$name])) {
+			return $this->getServiceType($this->meta[self::ALIASES][$name]);
+
+		} elseif (isset($this->meta[self::SERVICES][$name])) {
+			return $this->meta[self::SERVICES][$name];
+
+		} else {
+			throw new MissingServiceException("Service '$name' not found.");
+		}
+	}
+
+
 	/**
 	 * Does the service exist?
 	 * @param  string service name
@@ -112,8 +133,10 @@ class Container extends Nette\Object
 	 */
 	public function hasService($name)
 	{
+		$name = isset($this->meta[self::ALIASES][$name]) ? $this->meta[self::ALIASES][$name] : $name;
 		return isset($this->registry[$name])
-			|| method_exists($this, $method = Container::getMethodName($name)) && $this->getReflection()->getMethod($method)->getName() === $method;
+			|| (method_exists($this, $method = self::getMethodName($name))
+				&& ($rm = new \ReflectionMethod($this, $method)) && $rm->getName() === $method);
 	}
 
 
@@ -127,6 +150,7 @@ class Container extends Nette\Object
 		if (!$this->hasService($name)) {
 			throw new MissingServiceException("Service '$name' not found.");
 		}
+		$name = isset($this->meta[self::ALIASES][$name]) ? $this->meta[self::ALIASES][$name] : $name;
 		return isset($this->registry[$name]);
 	}
 
@@ -139,11 +163,12 @@ class Container extends Nette\Object
 	 */
 	public function createService($name, array $args = array())
 	{
-		$method = Container::getMethodName($name);
+		$name = isset($this->meta[self::ALIASES][$name]) ? $this->meta[self::ALIASES][$name] : $name;
+		$method = self::getMethodName($name);
 		if (isset($this->creating[$name])) {
 			throw new Nette\InvalidStateException(sprintf('Circular reference detected for services: %s.', implode(', ', array_keys($this->creating))));
 
-		} elseif (!method_exists($this, $method) || $this->getReflection()->getMethod($method)->getName() !== $method) {
+		} elseif (!method_exists($this, $method) || !($rm = new \ReflectionMethod($this, $method)) || $rm->getName() !== $method) {
 			throw new MissingServiceException("Service '$name' not found.");
 		}
 
@@ -173,15 +198,15 @@ class Container extends Nette\Object
 	 */
 	public function getByType($class, $need = TRUE)
 	{
-		$names = $this->findByType($class);
-		if (!$names) {
-			if ($need) {
-				throw new MissingServiceException("Service of type $class not found.");
+		$class = ltrim($class, '\\');
+		if (!empty($this->meta[self::TYPES][$class][TRUE])) {
+			if (count($names = $this->meta[self::TYPES][$class][TRUE]) === 1) {
+				return $this->getService($names[0]);
 			}
-		} elseif (count($names) > 1) {
 			throw new MissingServiceException("Multiple services of type $class found: " . implode(', ', $names) . '.');
-		} else {
-			return $this->getService($names[0]);
+
+		} elseif ($need) {
+			throw new MissingServiceException("Service of type $class not found.");
 		}
 	}
 
@@ -193,8 +218,10 @@ class Container extends Nette\Object
 	 */
 	public function findByType($class)
 	{
-		$class = ltrim(strtolower($class), '\\');
-		return isset($this->meta[self::TYPES][$class]) ? $this->meta[self::TYPES][$class] : array();
+		$class = ltrim($class, '\\');
+		return empty($this->meta[self::TYPES][$class])
+			? array()
+			: call_user_func_array('array_merge', $this->meta[self::TYPES][$class]);
 	}
 
 
@@ -221,7 +248,7 @@ class Container extends Nette\Object
 	 */
 	public function createInstance($class, array $args = array())
 	{
-		$rc = Nette\Reflection\ClassType::from($class);
+		$rc = new \ReflectionClass($class);
 		if (!$rc->isInstantiable()) {
 			throw new ServiceCreationException("Class $class is not instantiable.");
 
@@ -242,26 +269,12 @@ class Container extends Nette\Object
 	 */
 	public function callInjects($service)
 	{
-		if (!is_object($service)) {
-			throw new Nette\InvalidArgumentException(sprintf('Service must be object, %s given.', gettype($service)));
-		}
-
-		foreach (array_reverse(get_class_methods($service)) as $method) {
-			if (substr($method, 0, 6) === 'inject') {
-				$this->callMethod(array($service, $method));
-			}
-		}
-
-		foreach (Helpers::getInjectProperties(Nette\Reflection\ClassType::from($service), $this) as $property => $type) {
-			$service->$property = $this->getByType($type);
-		}
+		Extensions\InjectExtension::callInjects($this, $service);
 	}
 
 
 	/**
 	 * Calls method using autowiring.
-	 * @param  mixed   class, object, function, callable
-	 * @param  array   arguments
 	 * @return mixed
 	 */
 	public function callMethod($function, array $args = array())
@@ -280,6 +293,7 @@ class Container extends Nette\Object
 	 * Expands %placeholders%.
 	 * @param  mixed
 	 * @return mixed
+	 * @deprecated
 	 */
 	public function expand($s)
 	{
@@ -323,7 +337,7 @@ class Container extends Nette\Object
 	private function error($oldName, $newName)
 	{
 		if (empty($this->parameters['container']['accessors'])) {
-			trigger_error("$oldName() is deprecated; use $newName() or enable nette.container.accessors in configuration.", E_USER_DEPRECATED);
+			trigger_error("$oldName() is deprecated; use $newName() or enable di.accessors in configuration.", E_USER_DEPRECATED);
 		}
 	}
 
