@@ -1,8 +1,8 @@
 <?php
 
 /**
- * This file is part of the Nette Framework (http://nette.org)
- * Copyright (c) 2004 David Grudl (http://davidgrudl.com)
+ * This file is part of the Nette Framework (https://nette.org)
+ * Copyright (c) 2004 David Grudl (https://davidgrudl.com)
  */
 
 namespace Nette\Database\Drivers;
@@ -12,12 +12,16 @@ use Nette;
 
 /**
  * Supplemental ODBC database driver.
- *
- * @author     David Grudl
  */
 class OdbcDriver extends Nette\Object implements Nette\Database\ISupplementalDriver
 {
 
+	public function convertException(\PDOException $e)
+	{
+		return Nette\Database\DriverException::from($e);
+	}
+
+
 	/********************* SQL ****************d*g**/
 
 
@@ -48,6 +52,15 @@ class OdbcDriver extends Nette\Object implements Nette\Database\ISupplementalDri
 	}
 
 
+	/**
+	 * Formats date-time interval for use in a SQL statement.
+	 */
+	public function formatDateInterval(\DateInterval $value)
+	{
+		throw new Nette\NotSupportedException;
+	}
+
+
 	/**
 	 * Encodes string for use in a LIKE statement.
 	 */
@@ -63,16 +76,18 @@ class OdbcDriver extends Nette\Object implements Nette\Database\ISupplementalDri
 	 */
 	public function applyLimit(& $sql, $limit, $offset)
 	{
-		if ($limit >= 0) {
-			$sql = preg_replace('#^\s*(SELECT|UPDATE|DELETE)#i', '$0 TOP ' . (int) $limit, $sql, 1, $count);
+		if ($offset) {
+			throw new Nette\NotSupportedException('Offset is not supported by this database.');
+
+		} elseif ($limit < 0) {
+			throw new Nette\InvalidArgumentException('Negative offset or limit.');
+
+		} elseif ($limit !== NULL) {
+			$sql = preg_replace('#^\s*(SELECT(\s+DISTINCT|\s+ALL)?|UPDATE|DELETE)#i', '$0 TOP ' . (int) $limit, $sql, 1, $count);
 			if (!$count) {
 				throw new Nette\InvalidArgumentException('SQL query must begin with SELECT, UPDATE or DELETE command.');
 			}
 		}
-
-		if ($offset) {
-			throw new Nette\NotSupportedException('Offset is not supported by this database.');
-		}
 	}
 
 
@@ -134,6 +149,7 @@ class OdbcDriver extends Nette\Object implements Nette\Database\ISupplementalDri
 
 
 	/**
+	 * @param  string
 	 * @return bool
 	 */
 	public function isSupported($item)
