@@ -1,39 +1,31 @@
 <?php
 
 /**
- * This file is part of the Nette Framework (http://nette.org)
- * Copyright (c) 2004 David Grudl (http://davidgrudl.com)
+ * This file is part of the Nette Framework (https://nette.org)
+ * Copyright (c) 2004 David Grudl (https://davidgrudl.com)
  */
 
 namespace Nette\Forms\Controls;
 
-use Nette,
-	Nette\Forms\IControl,
-	Nette\Utils\Html,
-	Nette\Forms\Form,
-	Nette\Forms\Rule;
+use Nette;
+use Nette\Forms\IControl;
+use Nette\Utils\Html;
+use Nette\Forms\Form;
 
 
 /**
  * Base class that implements the basic functionality common to form controls.
  *
- * @author     David Grudl
- *
- * @property-read Nette\Forms\Form $form
+ * @property-read Form $form
  * @property-read string $htmlName
  * @property   string $htmlId
- * @property-read array $options
- * @property   Nette\Localization\ITranslator|NULL $translator
  * @property   mixed $value
- * @property-read bool $filled
- * @property-write $defaultValue
  * @property   bool $disabled
  * @property   bool $omitted
- * @property-read Nette\Utils\Html $control
- * @property-read Nette\Utils\Html $label
- * @property-read Nette\Utils\Html $controlPrototype
- * @property-read Nette\Utils\Html $labelPrototype
- * @property-read Nette\Forms\Rules $rules
+ * @property-read Html $control
+ * @property-read Html $label
+ * @property-read Html $controlPrototype
+ * @property-read Html $labelPrototype
  * @property   bool $required
  * @property-read array $errors
  */
@@ -48,10 +40,10 @@ abstract class BaseControl extends Nette\ComponentModel\Component implements ICo
 	/** @var mixed current control value */
 	protected $value;
 
-	/** @var Nette\Utils\Html  control element template */
+	/** @var Html  control element template */
 	protected $control;
 
-	/** @var Nette\Utils\Html  label element template */
+	/** @var Html  label element template */
 	protected $label;
 
 	/** @var array */
@@ -104,7 +96,7 @@ abstract class BaseControl extends Nette\ComponentModel\Component implements ICo
 	/**
 	 * Returns form.
 	 * @param  bool   throw exception if form doesn't exist?
-	 * @return Nette\Forms\Form
+	 * @return Form
 	 */
 	public function getForm($need = TRUE)
 	{
@@ -142,7 +134,7 @@ abstract class BaseControl extends Nette\ComponentModel\Component implements ICo
 	}
 
 
-	/********************* interface IFormControl ****************d*g**/
+	/********************* interface IControl ****************d*g**/
 
 
 	/**
@@ -243,20 +235,18 @@ abstract class BaseControl extends Nette\ComponentModel\Component implements ICo
 
 	/**
 	 * Generates control's HTML element.
-	 * @return Nette\Utils\Html|string
+	 * @return Html|string
 	 */
 	public function getControl()
 	{
 		$this->setOption('rendered', TRUE);
-
-		$rules = self::exportRules($this->rules);
 		$el = clone $this->control;
 		return $el->addAttributes(array(
 			'name' => $this->getHtmlName(),
 			'id' => $this->getHtmlId(),
 			'required' => $this->isRequired(),
 			'disabled' => $this->isDisabled(),
-			'data-nette-rules' => $rules ? Nette\Utils\Json::encode($rules) : NULL,
+			'data-nette-rules' => Nette\Forms\Helpers::exportRules($this->rules) ?: NULL,
 		));
 	}
 
@@ -264,7 +254,7 @@ abstract class BaseControl extends Nette\ComponentModel\Component implements ICo
 	/**
 	 * Generates label's HTML element.
 	 * @param  string
-	 * @return Nette\Utils\Html|string
+	 * @return Html|string
 	 */
 	public function getLabel($caption = NULL)
 	{
@@ -277,7 +267,7 @@ abstract class BaseControl extends Nette\ComponentModel\Component implements ICo
 
 	/**
 	 * Returns control's HTML element template.
-	 * @return Nette\Utils\Html
+	 * @return Html
 	 */
 	public function getControlPrototype()
 	{
@@ -287,7 +277,7 @@ abstract class BaseControl extends Nette\ComponentModel\Component implements ICo
 
 	/**
 	 * Returns label's HTML element template.
-	 * @return Nette\Utils\Html
+	 * @return Html
 	 */
 	public function getLabelPrototype()
 	{
@@ -314,7 +304,7 @@ abstract class BaseControl extends Nette\ComponentModel\Component implements ICo
 	public function getHtmlId()
 	{
 		if (!isset($this->control->id)) {
-			$this->control->id = sprintf(self::$idMask, $this->lookupPath(NULL));
+			$this->control->id = sprintf(self::$idMask, $this->lookupPath());
 		}
 		return $this->control->id;
 	}
@@ -372,7 +362,7 @@ abstract class BaseControl extends Nette\ComponentModel\Component implements ICo
 			$tmp = is_array($value) ? array(& $value) : array(array(& $value));
 			foreach ($tmp[0] as & $v) {
 				if ($v != NULL && !$v instanceof Html) { // intentionally ==
-					$v = $translator->translate((string) $v, $count);
+					$v = $translator->translate($v, $count);
 				}
 			}
 		}
@@ -390,9 +380,9 @@ abstract class BaseControl extends Nette\ComponentModel\Component implements ICo
 	 * @param  mixed      optional rule arguments
 	 * @return self
 	 */
-	public function addRule($operation, $message = NULL, $arg = NULL)
+	public function addRule($validator, $message = NULL, $arg = NULL)
 	{
-		$this->rules->addRule($operation, $message, $arg);
+		$this->rules->addRule($validator, $message, $arg);
 		return $this;
 	}
 
@@ -403,22 +393,22 @@ abstract class BaseControl extends Nette\ComponentModel\Component implements ICo
 	 * @param  mixed     optional condition arguments
 	 * @return Nette\Forms\Rules      new branch
 	 */
-	public function addCondition($operation, $value = NULL)
+	public function addCondition($validator, $value = NULL)
 	{
-		return $this->rules->addCondition($operation, $value);
+		return $this->rules->addCondition($validator, $value);
 	}
 
 
 	/**
 	 * Adds a validation condition based on another control a returns new branch.
-	 * @param  Nette\Forms\IControl form control
+	 * @param  IControl form control
 	 * @param  mixed      condition type
 	 * @param  mixed      optional condition arguments
 	 * @return Nette\Forms\Rules      new branch
 	 */
-	public function addConditionOn(IControl $control, $operation, $value = NULL)
+	public function addConditionOn(IControl $control, $validator, $value = NULL)
 	{
-		return $this->rules->addConditionOn($control, $operation, $value);
+		return $this->rules->addConditionOn($control, $validator, $value);
 	}
 
 
@@ -516,161 +506,11 @@ abstract class BaseControl extends Nette\ComponentModel\Component implements ICo
 	}
 
 
-	/**
-	 * @return array
-	 */
+	/** @deprecated */
 	protected static function exportRules($rules)
 	{
-		$payload = array();
-		foreach ($rules as $rule) {
-			if (!is_string($op = $rule->operation)) {
-				if (!Nette\Utils\Callback::isStatic($op)) {
-					continue;
-				}
-				$op = Nette\Utils\Callback::toString($op);
-			}
-			if ($rule->type === Rule::VALIDATOR) {
-				$item = array('op' => ($rule->isNegative ? '~' : '') . $op, 'msg' => $rules->formatMessage($rule, FALSE));
-
-			} elseif ($rule->type === Rule::CONDITION) {
-				$item = array(
-					'op' => ($rule->isNegative ? '~' : '') . $op,
-					'rules' => static::exportRules($rule->subRules),
-					'control' => $rule->control->getHtmlName()
-				);
-				if ($rule->subRules->getToggles()) {
-					$item['toggle'] = $rule->subRules->getToggles();
-				}
-			}
-
-			if (is_array($rule->arg)) {
-				foreach ($rule->arg as $key => $value) {
-					$item['arg'][$key] = $value instanceof IControl ? array('control' => $value->getHtmlName()) : $value;
-				}
-			} elseif ($rule->arg !== NULL) {
-				$item['arg'] = $rule->arg instanceof IControl ? array('control' => $rule->arg->getHtmlName()) : $rule->arg;
-			}
-
-			$payload[] = $item;
-		}
-		return $payload;
-	}
-
-
-	/********************* validators ****************d*g**/
-
-
-	/**
-	 * Is control's value equal with second parameter?
-	 * @return bool
-	 * @internal
-	 */
-	public static function validateEqual(IControl $control, $arg)
-	{
-		$value = $control->getValue();
-		foreach ((is_array($value) ? $value : array($value)) as $val) {
-			foreach ((is_array($arg) ? $arg : array($arg)) as $item) {
-				if ((string) $val === (string) $item) {
-					continue 2;
-				}
-			}
-			return FALSE;
-		}
-		return TRUE;
-	}
-
-
-	/**
-	 * Is control's value not equal with second parameter?
-	 * @return bool
-	 * @internal
-	 */
-	public static function validateNotEqual(IControl $control, $arg)
-	{
-		return !static::validateEqual($control, $arg);
-	}
-
-
-	/**
-	 * Is control filled?
-	 * @return bool
-	 * @internal
-	 */
-	public static function validateFilled(IControl $control)
-	{
-		return $control->isFilled();
-	}
-
-
-	/**
-	 * Is control not filled?
-	 * @return bool
-	 * @internal
-	 */
-	public static function validateBlank(IControl $control)
-	{
-		return !$control->isFilled();
-	}
-
-
-	/**
-	 * Is control valid?
-	 * @return bool
-	 * @internal
-	 */
-	public static function validateValid(IControl $control)
-	{
-		return $control->rules->validate();
-	}
-
-
-	/**
-	 * Is a control's value number in specified range?
-	 * @param  Nette\Forms\IControl
-	 * @param  array  min and max value pair
-	 * @return bool
-	 * @internal
-	 */
-	public static function validateRange(IControl $control, $range)
-	{
-		return Nette\Utils\Validators::isInRange($control->getValue(), $range);
-	}
-
-
-	/**
-	 * Count/length validator. Range is array, min and max length pair.
-	 * @return bool
-	 * @internal
-	 */
-	public static function validateLength(IControl $control, $range)
-	{
-		if (!is_array($range)) {
-			$range = array($range, $range);
-		}
-		$value = $control->getValue();
-		return Nette\Utils\Validators::isInRange(is_array($value) ? count($value) : Nette\Utils\Strings::length($value), $range);
-	}
-
-
-	/**
-	 * Has control's value minimal count/length?
-	 * @return bool
-	 * @internal
-	 */
-	public static function validateMinLength(IControl $control, $length)
-	{
-		return static::validateLength($control, array($length, NULL));
-	}
-
-
-	/**
-	 * Is control's value count/length in limit?
-	 * @return bool
-	 * @internal
-	 */
-	public static function validateMaxLength(IControl $control, $length)
-	{
-		return static::validateLength($control, array(NULL, $length));
+		trigger_error(__METHOD__ . '() is deprecated; use Nette\Forms\Helpers::exportRules() instead.', E_USER_DEPRECATED);
+		return Nette\Forms\Helpers::exportRules($rules);
 	}
 
 
