@@ -1,32 +1,37 @@
 <?php
 
 /**
- * This file is part of the Nette Framework (http://nette.org)
- * Copyright (c) 2004 David Grudl (http://davidgrudl.com)
+ * This file is part of the Nette Framework (https://nette.org)
+ * Copyright (c) 2004 David Grudl (https://davidgrudl.com)
  */
 
 namespace Nette\Forms\Controls;
 
-use Nette,
-	Nette\Utils\Html;
+use Nette;
+use Nette\Utils\Html;
 
 
 /**
  * Set of radio button controls.
  *
- * @author     David Grudl
- *
- * @property-read Nette\Utils\Html $separatorPrototype
- * @property-read Nette\Utils\Html $containerPrototype
+ * @property-read Html $separatorPrototype
+ * @property-read Html $containerPrototype
+ * @property-read Html $itemLabelPrototype
  */
 class RadioList extends ChoiceControl
 {
-	/** @var Nette\Utils\Html  separator element template */
+	/** @var bool */
+	public $generateId = FALSE;
+
+	/** @var Html  separator element template */
 	protected $separator;
 
-	/** @var Nette\Utils\Html  container element template */
+	/** @var Html  container element template */
 	protected $container;
 
+	/** @var Html  item label template */
+	protected $itemLabel;
+
 
 	/**
 	 * @param  string  label
@@ -38,70 +43,35 @@ class RadioList extends ChoiceControl
 		$this->control->type = 'radio';
 		$this->container = Html::el();
 		$this->separator = Html::el('br');
-	}
-
-
-	/**
-	 * Returns selected radio value.
-	 * @return mixed
-	 */
-	public function getValue($raw = FALSE)
-	{
-		if ($raw) {
-			trigger_error(__METHOD__ . '(TRUE) is deprecated; use getRawValue() instead.', E_USER_DEPRECATED);
-			return $this->getRawValue();
-		}
-		return parent::getValue();
-	}
-
-
-	/**
-	 * Returns separator HTML element template.
-	 * @return Nette\Utils\Html
-	 */
-	public function getSeparatorPrototype()
-	{
-		return $this->separator;
-	}
-
-
-	/**
-	 * Returns container HTML element template.
-	 * @return Nette\Utils\Html
-	 */
-	public function getContainerPrototype()
-	{
-		return $this->container;
+		$this->itemLabel = Html::el();
 	}
 
 
 	/**
 	 * Generates control's HTML element.
-	 * @return Nette\Utils\Html
+	 * @return Html
 	 */
-	public function getControl($key = NULL)
+	public function getControl()
 	{
-		if ($key !== NULL) {
-			trigger_error(sprintf('Partial %s() is deprecated; use getControlPart() instead.', __METHOD__), E_USER_DEPRECATED);
-			return $this->getControlPart($key);
-		}
-
 		$input = parent::getControl();
+		$items = $this->getItems();
 		$ids = array();
-		foreach ($this->getItems() as $value => $label) {
-			$ids[$value] = $input->id . '-' . $value;
+		if ($this->generateId) {
+			foreach ($items as $value => $label) {
+				$ids[$value] = $input->id . '-' . $value;
+			}
 		}
 
 		return $this->container->setHtml(
 			Nette\Forms\Helpers::createInputList(
-				$this->translate($this->getItems()),
+				$this->translate($items),
 				array_merge($input->attrs, array(
 					'id:' => $ids,
 					'checked?' => $this->value,
 					'disabled:' => $this->disabled,
-					'data-nette-rules:' => array(key($ids) => $input->attrs['data-nette-rules']),
+					'data-nette-rules:' => array(key($items) => $input->attrs['data-nette-rules']),
 				)),
-				array('for:' => $ids),
+				array('for:' => $ids) + $this->itemLabel->attrs,
 				$this->separator
 			)
 		);
@@ -111,23 +81,20 @@ class RadioList extends ChoiceControl
 	/**
 	 * Generates label's HTML element.
 	 * @param  string
-	 * @return Nette\Utils\Html
+	 * @return Html
 	 */
-	public function getLabel($caption = NULL, $key = NULL)
+	public function getLabel($caption = NULL)
 	{
-		if ($key !== NULL) {
-			trigger_error(sprintf('Partial %s() is deprecated; use getLabelPart() instead.', __METHOD__), E_USER_DEPRECATED);
-			return $this->getLabelPart($key);
-		}
 		return parent::getLabel($caption)->for(NULL);
 	}
 
 
 	/**
-	 * @return Nette\Utils\Html
+	 * @return Html
 	 */
 	public function getControlPart($key)
 	{
+		$key = key(array((string) $key => NULL));
 		return parent::getControl()->addAttributes(array(
 			'id' => $this->getHtmlId() . '-' . $key,
 			'checked' => in_array($key, (array) $this->value, TRUE),
@@ -138,11 +105,43 @@ class RadioList extends ChoiceControl
 
 
 	/**
-	 * @return Nette\Utils\Html
+	 * @return Html
 	 */
-	public function getLabelPart($key)
+	public function getLabelPart($key = NULL)
 	{
-		return parent::getLabel($this->items[$key])->for($this->getHtmlId() . '-' . $key);
+		return func_num_args()
+			? parent::getLabel($this->items[$key])->for($this->getHtmlId() . '-' . $key)
+			: $this->getLabel();
+	}
+
+
+	/**
+	 * Returns separator HTML element template.
+	 * @return Html
+	 */
+	public function getSeparatorPrototype()
+	{
+		return $this->separator;
+	}
+
+
+	/**
+	 * Returns container HTML element template.
+	 * @return Html
+	 */
+	public function getContainerPrototype()
+	{
+		return $this->container;
+	}
+
+
+	/**
+	 * Returns item label HTML element template.
+	 * @return Html
+	 */
+	public function getItemLabelPrototype()
+	{
+		return $this->itemLabel;
 	}
 
 }
