@@ -1,8 +1,8 @@
 <?php
 
 /**
- * This file is part of the Nette Framework (http://nette.org)
- * Copyright (c) 2004 David Grudl (http://davidgrudl.com)
+ * This file is part of the Nette Framework (https://nette.org)
+ * Copyright (c) 2004 David Grudl (https://davidgrudl.com)
  */
 
 namespace Nette\Forms;
@@ -13,26 +13,16 @@ use Nette;
 /**
  * Creates, validates and renders HTML forms.
  *
- * @author     David Grudl
- *
- * @property   mixed $action
- * @property   string $method
- * @property-read array $groups
- * @property   Nette\Localization\ITranslator|NULL $translator
- * @property-read bool $anchored
- * @property-read ISubmitterControl|FALSE $submitted
- * @property-read bool $success
- * @property-read array $httpData
  * @property-read array $errors
  * @property-read Nette\Utils\Html $elementPrototype
- * @property   IFormRenderer $renderer
  */
-class Form extends Container
+class Form extends Container implements Nette\Utils\IHtmlString
 {
 	/** validator */
 	const EQUAL = ':equal',
 		IS_IN = self::EQUAL,
 		NOT_EQUAL = ':notEqual',
+		IS_NOT_IN = self::NOT_EQUAL,
 		FILLED = ':filled',
 		BLANK = ':blank',
 		REQUIRED = self::FILLED,
@@ -50,11 +40,12 @@ class Form extends Container
 		LENGTH = ':length',
 		EMAIL = ':email',
 		URL = ':url',
-		REGEXP = ':regexp',
 		PATTERN = ':pattern',
 		INTEGER = ':integer',
 		NUMERIC = ':integer',
 		FLOAT = ':float',
+		MIN = ':min',
+		MAX = ':max',
 		RANGE = ':range';
 
 	// multiselect
@@ -82,13 +73,13 @@ class Form extends Container
 	/** @internal protection token ID */
 	const PROTECTOR_ID = '_token_';
 
-	/** @var array of function(Form $sender); Occurs when the form is submitted and successfully validated */
+	/** @var callable[]  function (Form $sender); Occurs when the form is submitted and successfully validated */
 	public $onSuccess;
 
-	/** @var array of function(Form $sender); Occurs when the form is submitted and is not valid */
+	/** @var callable[]  function (Form $sender); Occurs when the form is submitted and is not valid */
 	public $onError;
 
-	/** @var array of function(Form $sender); Occurs when the form is submitted */
+	/** @var callable[]  function (Form $sender); Occurs when the form is submitted */
 	public $onSubmit;
 
 	/** @var mixed or NULL meaning: not detected yet */
@@ -122,18 +113,24 @@ class Form extends Container
 	 */
 	public function __construct($name = NULL)
 	{
-		$this->element = Nette\Utils\Html::el('form');
-		$this->element->action = ''; // RFC 1808 -> empty uri means 'this'
-		$this->element->method = self::POST;
-		$this->element->id = $name === NULL ? NULL : 'frm-' . $name;
-
-		$this->monitor(__CLASS__);
+		parent::__construct();
 		if ($name !== NULL) {
+			$this->getElementPrototype()->id = 'frm-' . $name;
 			$tracker = new Controls\HiddenField($name);
 			$tracker->setOmitted();
 			$this[self::TRACKER_ID] = $tracker;
+			$this->setParent(NULL, $name);
 		}
-		parent::__construct(NULL, $name);
+	}
+
+
+	/**
+	 * @return void
+	 */
+	protected function validateParent(Nette\ComponentModel\IContainer $parent)
+	{
+		parent::validateParent($parent);
+		$this->monitor(__CLASS__);
 	}
 
 
@@ -153,7 +150,7 @@ class Form extends Container
 
 	/**
 	 * Returns self.
-	 * @return Form
+	 * @return self
 	 */
 	public function getForm($need = TRUE)
 	{
@@ -168,7 +165,7 @@ class Form extends Container
 	 */
 	public function setAction($url)
 	{
-		$this->element->action = $url;
+		$this->getElementPrototype()->action = $url;
 		return $this;
 	}
 
@@ -179,7 +176,7 @@ class Form extends Container
 	 */
 	public function getAction()
 	{
-		return $this->element->action;
+		return $this->getElementPrototype()->action;
 	}
 
 
@@ -193,7 +190,7 @@ class Form extends Container
 		if ($this->httpData !== NULL) {
 			throw new Nette\InvalidStateException(__METHOD__ . '() must be called until the form is empty.');
 		}
-		$this->element->method = strtolower($method);
+		$this->getElementPrototype()->method = strtolower($method);
 		return $this;
 	}
 
@@ -204,7 +201,7 @@ class Form extends Container
 	 */
 	public function getMethod()
 	{
-		return $this->element->method;
+		return $this->getElementPrototype()->method;
 	}
 
 
@@ -403,17 +400,24 @@ class Form extends Container
 			}
 		}
 
-		if ($this->onSuccess) {
+		if (!$this->isValid()) {
+			$this->onError($this);
+
+		} elseif ($this->onSuccess !== NULL) {
+			if (!is_array($this->onSuccess) && !$this->onSuccess instanceof \Traversable) {
+				throw new Nette\UnexpectedValueException('Property Form::$onSuccess must be array or Traversable, ' . gettype($this->onSuccess) . ' given.');
+			}
 			foreach ($this->onSuccess as $handler) {
+				$params = Nette\Utils\Callback::toReflection($handler)->getParameters();
+				$values = isset($params[1]) ? $this->getValues($params[1]->isArray()) : NULL;
+				Nette\Utils\Callback::invoke($handler, $this, $values);
 				if (!$this->isValid()) {
 					$this->onError($this);
 					break;
 				}
-				Nette\Utils\Callback::invoke($handler, $this);
 			}
-		} elseif (!$this->isValid()) {
-			$this->onError($this);
 		}
+
 		$this->onSubmit($this);
 	}
 
@@ -474,7 +478,7 @@ class Form extends Container
 			$maxSize <<= $units[$ch];
 		}
 		if ($maxSize > 0 && $maxSize < $_SERVER['CONTENT_LENGTH']) {
-			$this->addError(sprintf(Rules::$defaultMessages[self::MAX_FILE_SIZE], $maxSize));
+			$this->addError(sprintf(Validator::$messages[self::MAX_FILE_SIZE], $maxSize));
 		}
 	}
 
@@ -528,14 +532,6 @@ class Form extends Container
 	}
 
 
-	/** @deprecated */
-	public function getAllErrors()
-	{
-		trigger_error(__METHOD__ . '() is deprecated; use getErrors() instead.', E_USER_DEPRECATED);
-		return $this->getErrors();
-	}
-
-
 	/********************* rendering ****************d*g**/
 
 
@@ -545,6 +541,11 @@ class Form extends Container
 	 */
 	public function getElementPrototype()
 	{
+		if (!$this->element) {
+			$this->element = Nette\Utils\Html::el('form');
+			$this->element->action = ''; // RFC 1808 -> empty uri means 'this'
+			$this->element->method = self::POST;
+		}
 		return $this->element;
 	}
 
@@ -587,7 +588,7 @@ class Form extends Container
 
 	/**
 	 * Renders form to string.
-	 * @return can throw exceptions? (hidden parameter)
+	 * @param can throw exceptions? (hidden parameter)
 	 * @return string
 	 */
 	public function __toString()
@@ -595,7 +596,10 @@ class Form extends Container
 		try {
 			return $this->getRenderer()->render($this);
 
+		} catch (\Throwable $e) {
 		} catch (\Exception $e) {
+		}
+		if (isset($e)) {
 			if (func_num_args()) {
 				throw $e;
 			}
