@@ -1,19 +1,15 @@
 <?php
 
 /**
- * This file is part of the Nette Framework (http://nette.org)
- * Copyright (c) 2004 David Grudl (http://davidgrudl.com)
+ * This file is part of the Nette Framework (https://nette.org)
+ * Copyright (c) 2004 David Grudl (https://davidgrudl.com)
  */
 
 namespace Nette\Http;
 
-use Nette;
-
 
 /**
  * IHttpResponse interface.
- *
- * @author     David Grudl
  */
 interface IResponse
 {
@@ -25,24 +21,48 @@ interface IResponse
 
 	/** HTTP 1.1 response code */
 	const
+		S100_CONTINUE = 100,
+		S101_SWITCHING_PROTOCOLS = 101,
 		S200_OK = 200,
+		S201_CREATED = 201,
+		S202_ACCEPTED = 202,
+		S203_NON_AUTHORITATIVE_INFORMATION = 203,
 		S204_NO_CONTENT = 204,
+		S205_RESET_CONTENT = 205,
+		S206_PARTIAL_CONTENT = 206,
 		S300_MULTIPLE_CHOICES = 300,
 		S301_MOVED_PERMANENTLY = 301,
 		S302_FOUND = 302,
 		S303_SEE_OTHER = 303,
 		S303_POST_GET = 303,
 		S304_NOT_MODIFIED = 304,
-		S307_TEMPORARY_REDIRECT= 307,
+		S305_USE_PROXY = 305,
+		S307_TEMPORARY_REDIRECT = 307,
 		S400_BAD_REQUEST = 400,
 		S401_UNAUTHORIZED = 401,
+		S402_PAYMENT_REQUIRED = 402,
 		S403_FORBIDDEN = 403,
 		S404_NOT_FOUND = 404,
 		S405_METHOD_NOT_ALLOWED = 405,
+		S406_NOT_ACCEPTABLE = 406,
+		S407_PROXY_AUTHENTICATION_REQUIRED = 407,
+		S408_REQUEST_TIMEOUT = 408,
+		S409_CONFLICT = 409,
 		S410_GONE = 410,
+		S411_LENGTH_REQUIRED = 411,
+		S412_PRECONDITION_FAILED = 412,
+		S413_REQUEST_ENTITY_TOO_LARGE = 413,
+		S414_REQUEST_URI_TOO_LONG = 414,
+		S415_UNSUPPORTED_MEDIA_TYPE = 415,
+		S416_REQUESTED_RANGE_NOT_SATISFIABLE = 416,
+		S417_EXPECTATION_FAILED = 417,
+		S426_UPGRADE_REQUIRED = 426,
 		S500_INTERNAL_SERVER_ERROR = 500,
 		S501_NOT_IMPLEMENTED = 501,
-		S503_SERVICE_UNAVAILABLE = 503;
+		S502_BAD_GATEWAY = 502,
+		S503_SERVICE_UNAVAILABLE = 503,
+		S504_GATEWAY_TIMEOUT = 504,
+		S505_HTTP_VERSION_NOT_SUPPORTED = 505;
 
 	/**
 	 * Sets HTTP response code.
@@ -91,7 +111,7 @@ interface IResponse
 
 	/**
 	 * Sets the number of seconds before a page cached on a browser expires.
-	 * @param  mixed  timestamp or number of seconds
+	 * @param  string|int|\DateTime  time, value 0 means "until the browser is closed"
 	 * @return void
 	 */
 	function setExpiration($seconds);
@@ -102,9 +122,17 @@ interface IResponse
 	 */
 	function isSent();
 
+	/**
+	 * Returns value of an HTTP header.
+	 * @param  string
+	 * @param  mixed
+	 * @return mixed
+	 */
+	function getHeader($header, $default = NULL);
+
 	/**
 	 * Returns a list of headers to sent.
-	 * @return array
+	 * @return array (name => value)
 	 */
 	function getHeaders();
 
