@@ -1,23 +1,18 @@
 <?php
 
 /**
- * This file is part of the Nette Framework (http://nette.org)
- * Copyright (c) 2004 David Grudl (http://davidgrudl.com)
+ * This file is part of the Nette Framework (https://nette.org)
+ * Copyright (c) 2004 David Grudl (https://davidgrudl.com)
  */
 
 namespace Nette\Http;
 
 use Nette;
+use Nette\Utils\DateTime;
 
 
 /**
  * HttpResponse class.
- *
- * @author     David Grudl
- *
- * @property   int $code
- * @property-read bool $sent
- * @property-read array $headers
  */
 class Response extends Nette\Object implements IResponse
 {
@@ -46,10 +41,14 @@ class Response extends Nette\Object implements IResponse
 	public function __construct()
 	{
 		if (PHP_VERSION_ID >= 50400) {
-			if (is_int(http_response_code())) {
-				$this->code = http_response_code();
+			if (is_int($code = http_response_code())) {
+				$this->code = $code;
 			}
-			header_register_callback($this->removeDuplicateCookies);
+		}
+
+		if (PHP_VERSION_ID >= 50401) { // PHP bug #61106
+			$rm = new \ReflectionMethod('Nette\Http\Helpers::removeDuplicateCookies');
+			header_register_callback($rm->getClosure()); // requires closure due PHP bug #66375
 		}
 	}
 
@@ -146,13 +145,16 @@ class Response extends Nette\Object implements IResponse
 	{
 		$this->setCode($code);
 		$this->setHeader('Location', $url);
-		echo "<h1>Redirect</h1>\n\n<p><a href=\"" . htmlSpecialChars($url, ENT_IGNORE | ENT_QUOTES) . "\">Please click here to continue</a>.</p>";
+		if (preg_match('#^https?:|^\s*+[a-z0-9+.-]*+[^:]#i', $url)) {
+			$escapedUrl = htmlSpecialChars($url, ENT_IGNORE | ENT_QUOTES, 'UTF-8');
+			echo "<h1>Redirect</h1>\n\n<p><a href=\"$escapedUrl\">Please click here to continue</a>.</p>";
+		}
 	}
 
 
 	/**
 	 * Sets the number of seconds before a page cached on a browser expires.
-	 * @param  string|int|DateTime  time, value 0 means "until the browser is closed"
+	 * @param  string|int|\DateTime  time, value 0 means "until the browser is closed"
 	 * @return self
 	 * @throws Nette\InvalidStateException  if HTTP headers have been sent
 	 */
@@ -164,9 +166,9 @@ class Response extends Nette\Object implements IResponse
 			return $this;
 		}
 
-		$time = Nette\DateTime::from($time);
+		$time = DateTime::from($time);
 		$this->setHeader('Cache-Control', 'max-age=' . ($time->format('U') - time()));
-		$this->setHeader('Expires', self::date($time));
+		$this->setHeader('Expires', Helpers::formatDate($time));
 		return $this;
 	}
 
@@ -182,7 +184,7 @@ class Response extends Nette\Object implements IResponse
 
 
 	/**
-	 * Return the value of the HTTP header.
+	 * Returns value of an HTTP header.
 	 * @param  string
 	 * @param  mixed
 	 * @return mixed
@@ -202,7 +204,7 @@ class Response extends Nette\Object implements IResponse
 
 	/**
 	 * Returns a list of headers to sent.
-	 * @return array
+	 * @return array (name => value)
 	 */
 	public function getHeaders()
 	{
@@ -216,15 +218,11 @@ class Response extends Nette\Object implements IResponse
 
 
 	/**
-	 * Returns HTTP valid date format.
-	 * @param  string|int|DateTime
-	 * @return string
+	 * @deprecated
 	 */
 	public static function date($time = NULL)
 	{
-		$time = Nette\DateTime::from($time);
-		$time->setTimezone(new \DateTimeZone('GMT'));
-		return $time->format('D, d M Y H:i:s \G\M\T');
+		return Helpers::formatDate($time);
 	}
 
 
@@ -237,7 +235,7 @@ class Response extends Nette\Object implements IResponse
 			&& in_array($this->code, array(400, 403, 404, 405, 406, 408, 409, 410, 500, 501, 505), TRUE)
 			&& preg_match('#^text/html(?:;|$)#', $this->getHeader('Content-Type', 'text/html'))
 		) {
-			echo Nette\Utils\Strings::random(2e3, " \t\r\n"); // sends invisible garbage for IE
+			echo Nette\Utils\Random::generate(2e3, " \t\r\n"); // sends invisible garbage for IE
 			self::$fixIE = FALSE;
 		}
 	}
@@ -247,7 +245,7 @@ class Response extends Nette\Object implements IResponse
 	 * Sends a cookie.
 	 * @param  string name of the cookie
 	 * @param  string value
-	 * @param  string|int|DateTime  expiration time, value 0 means "until the browser is closed"
+	 * @param  string|int|\DateTime  expiration time, value 0 means "until the browser is closed"
 	 * @param  string
 	 * @param  string
 	 * @param  bool
@@ -261,13 +259,13 @@ class Response extends Nette\Object implements IResponse
 		setcookie(
 			$name,
 			$value,
-			$time ? Nette\DateTime::from($time)->format('U') : 0,
+			$time ? DateTime::from($time)->format('U') : 0,
 			$path === NULL ? $this->cookiePath : (string) $path,
 			$domain === NULL ? $this->cookieDomain : (string) $domain,
 			$secure === NULL ? $this->cookieSecure : (bool) $secure,
 			$httpOnly === NULL ? $this->cookieHttpOnly : (bool) $httpOnly
 		);
-		$this->removeDuplicateCookies();
+		Helpers::removeDuplicateCookies();
 		return $this;
 	}
 
@@ -287,27 +285,10 @@ class Response extends Nette\Object implements IResponse
 	}
 
 
-	/**
-	 * Removes duplicate cookies from response.
-	 * @return void
-	 * @internal
-	 */
+	/** @internal @deprecated */
 	public function removeDuplicateCookies()
 	{
-		if (headers_sent($file, $line) || ini_get('suhosin.cookie.encrypt')) {
-			return;
-		}
-
-		$flatten = array();
-		foreach (headers_list() as $header) {
-			if (preg_match('#^Set-Cookie: .+?=#', $header, $m)) {
-				$flatten[$m[0]] = $header;
-				header_remove('Set-Cookie');
-			}
-		}
-		foreach (array_values($flatten) as $key => $header) {
-			header($header, $key === 0);
-		}
+		trigger_error('Use Nette\Http\Helpers::removeDuplicateCookies()', E_USER_WARNING);
 	}
 
 
@@ -315,8 +296,9 @@ class Response extends Nette\Object implements IResponse
 	{
 		if (headers_sent($file, $line)) {
 			throw new Nette\InvalidStateException('Cannot send header after HTTP headers have been sent' . ($file ? " (output started at $file:$line)." : '.'));
-		} elseif ($this->warnOnBuffer && ob_get_length() && !array_filter(ob_get_status(TRUE), function($i) { return !$i['chunk_size']; })) {
-			trigger_error('Possible problem: you are sending a HTTP header while already having some data in output buffer. Try OutputDebugger or start session earlier.', E_USER_NOTICE);
+
+		} elseif ($this->warnOnBuffer && ob_get_length() && !array_filter(ob_get_status(TRUE), function ($i) { return !$i['chunk_size']; })) {
+			trigger_error('Possible problem: you are sending a HTTP header while already having some data in output buffer. Try Tracy\OutputDebugger or start session earlier.', E_USER_NOTICE);
 		}
 	}
 
