@@ -1,21 +1,18 @@
 <?php
 
 /**
- * This file is part of the Nette Framework (http://nette.org)
- * Copyright (c) 2004 David Grudl (http://davidgrudl.com)
+ * This file is part of the Nette Framework (https://nette.org)
+ * Copyright (c) 2004 David Grudl (https://davidgrudl.com)
  */
 
 namespace Nette\Http;
 
-use Nette,
-	Nette\Security\IIdentity;
+use Nette;
+use Nette\Security\IIdentity;
 
 
 /**
  * Session storage for user object.
- *
- * @author  David Grudl
- * @author  Jan Tichý
  */
 class UserStorage extends Nette\Object implements Nette\Security\IUserStorage
 {
@@ -29,7 +26,7 @@ class UserStorage extends Nette\Object implements Nette\Security\IUserStorage
 	private $sessionSection;
 
 
-	public function  __construct(Session $sessionHandler)
+	public function __construct(Session $sessionHandler)
 	{
 		$this->sessionHandler = $sessionHandler;
 	}
@@ -120,7 +117,7 @@ class UserStorage extends Nette\Object implements Nette\Security\IUserStorage
 
 	/**
 	 * Enables log out after inactivity.
-	 * @param  string|int|DateTime Number of seconds or timestamp
+	 * @param  string|int|\DateTime Number of seconds or timestamp
 	 * @param  int Log out when the browser is closed | Clear the identity from persistent storage?
 	 * @return self
 	 */
@@ -128,7 +125,7 @@ class UserStorage extends Nette\Object implements Nette\Security\IUserStorage
 	{
 		$section = $this->getSessionSection(TRUE);
 		if ($time) {
-			$time = Nette\DateTime::from($time)->format('U');
+			$time = Nette\Utils\DateTime::from($time)->format('U');
 			$section->expireTime = $time;
 			$section->expireDelta = $time - time();
 
@@ -147,7 +144,7 @@ class UserStorage extends Nette\Object implements Nette\Security\IUserStorage
 
 	/**
 	 * Why was user logged out?
-	 * @return int
+	 * @return int|NULL
 	 */
 	public function getLogoutReason()
 	{
