@@ -1,17 +1,16 @@
 <?php
 
 /**
- * This file is part of the Nette Framework (http://nette.org)
- * Copyright (c) 2004 David Grudl (http://davidgrudl.com)
+ * This file is part of the Latte (https://latte.nette.org)
+ * Copyright (c) 2008 David Grudl (https://davidgrudl.com)
  */
 
-namespace Nette\Latte\Macros;
+namespace Latte\Macros;
 
-use Nette,
-	Nette\Latte,
-	Nette\Latte\CompileException,
-	Nette\Latte\MacroNode,
-	Nette\Latte\PhpWriter;
+use Latte;
+use Latte\CompileException;
+use Latte\MacroNode;
+use Latte\PhpWriter;
 
 
 /**
@@ -22,21 +21,18 @@ use Nette,
  * - {for ?} ... {/for}
  * - {foreach ?} ... {/foreach}
  * - {$variable} with escaping
- * - {!$variable} without escaping
  * - {=expression} echo with escaping
- * - {!=expression} echo without escaping
  * - {?expression} evaluate PHP statement
  * - {_expression} echo translation with escaping
- * - {!_expression} echo translation without escaping
  * - {attr ?} HTML element attributes
  * - {capture ?} ... {/capture} capture block to parameter
  * - {var var => value} set template parameter
  * - {default var => value} set default template parameter
  * - {dump $var}
  * - {debugbreak}
+ * - {contentType ...} HTTP Content-Type header
+ * - {status ...} HTTP status
  * - {l} {r} to display { }
- *
- * @author     David Grudl
  */
 class CoreMacros extends MacroSet
 {
@@ -66,7 +62,6 @@ class CoreMacros extends MacroSet
 		$me->addMacro('sep', 'if (!$iterator->isLast(%node.args)) {', '}');
 
 		$me->addMacro('var', array($me, 'macroVar'));
-		$me->addMacro('assign', array($me, 'macroVar')); // deprecated
 		$me->addMacro('default', array($me, 'macroVar'));
 		$me->addMacro('dump', array($me, 'macroDump'));
 		$me->addMacro('debugbreak', array($me, 'macroDebugbreak'));
@@ -80,10 +75,12 @@ class CoreMacros extends MacroSet
 		$me->addMacro('capture', array($me, 'macroCapture'), array($me, 'macroCaptureEnd'));
 		$me->addMacro('include', array($me, 'macroInclude'));
 		$me->addMacro('use', array($me, 'macroUse'));
+		$me->addMacro('contentType', array($me, 'macroContentType'));
+		$me->addMacro('status', array($me, 'macroStatus'));
+		$me->addMacro('php', array($me, 'macroExpr'));
 
 		$me->addMacro('class', NULL, NULL, array($me, 'macroClass'));
-		$me->addMacro('attr', array($me, 'macroOldAttr'), '', array($me, 'macroAttr'));
-		$me->addMacro('href', NULL); // placeholder
+		$me->addMacro('attr', NULL, NULL, array($me, 'macroAttr'));
 	}
 
 
@@ -93,8 +90,10 @@ class CoreMacros extends MacroSet
 	 */
 	public function finalize()
 	{
-		return array('list($_l, $_g) = Nette\Latte\Macros\CoreMacros::initRuntime($template, '
-			. var_export($this->getCompiler()->getTemplateId(), TRUE) . ')');
+		return array('list($_b, $_g, $_l) = $template->initialize('
+			. var_export($this->getCompiler()->getTemplateId(), TRUE) . ', '
+			. var_export($this->getCompiler()->getContentType(), TRUE)
+		. ')');
 	}
 
 
@@ -106,8 +105,11 @@ class CoreMacros extends MacroSet
 	 */
 	public function macroIf(MacroNode $node, PhpWriter $writer)
 	{
+		if ($node->modifiers) {
+			trigger_error("Modifiers are not allowed in {{$node->name}}", E_USER_WARNING);
+		}
 		if ($node->data->capture = ($node->args === '')) {
-			return 'ob_start()';
+			return 'ob_start(function () {})';
 		}
 		if ($node->prefix === $node::PREFIX_TAG) {
 			return $writer->write($node->htmlNode->closing ? 'if (array_pop($_l->ifs)) {' : 'if ($_l->ifs[] = (%node.args)) {');
@@ -126,9 +128,9 @@ class CoreMacros extends MacroSet
 				throw new CompileException('Missing condition in {if} macro.');
 			}
 			return $writer->write('if (%node.args) '
-				. (isset($node->data->else) ? '{ ob_end_clean(); ob_end_flush(); }' : 'ob_end_flush();')
+				. (isset($node->data->else) ? '{ ob_end_clean(); echo ob_get_clean(); }' : 'echo ob_get_clean();')
 				. ' else '
-				. (isset($node->data->else) ? '{ $_else = ob_get_contents(); ob_end_clean(); ob_end_clean(); echo $_else; }' : 'ob_end_clean();')
+				. (isset($node->data->else) ? '{ $_l->else = ob_get_clean(); ob_end_clean(); echo $_l->else; }' : 'ob_end_clean();')
 			);
 		}
 		return '}';
@@ -140,13 +142,19 @@ class CoreMacros extends MacroSet
 	 */
 	public function macroElse(MacroNode $node, PhpWriter $writer)
 	{
+		if ($node->modifiers) {
+			trigger_error("Modifiers are not allowed in {{$node->name}}", E_USER_WARNING);
+		} elseif ($node->args) {
+			$hint = substr($node->args, 0, 2) === 'if' ? ', did you mean {elseif}?' : '';
+			trigger_error("Arguments are not allowed in {{$node->name}}$hint", E_USER_WARNING);
+		}
 		$ifNode = $node->parentNode;
 		if ($ifNode && $ifNode->name === 'if' && $ifNode->data->capture) {
 			if (isset($ifNode->data->else)) {
 				throw new CompileException('Macro {if} supports only one {else}.');
 			}
 			$ifNode->data->else = TRUE;
-			return 'ob_start()';
+			return 'ob_start(function () {})';
 		}
 		return '} else {';
 	}
@@ -163,7 +171,7 @@ class CoreMacros extends MacroSet
 			throw new CompileException("Unknown attribute n:{$node->prefix}-{$node->name}, use n:{$node->name} attribute.");
 		}
 
-		return $writer->write('ob_start()');
+		return $writer->write('ob_start(function () {})');
 	}
 
 
@@ -176,9 +184,9 @@ class CoreMacros extends MacroSet
 		$node->content = $parts[1]
 			. '<?php ob_start() ?>'
 			. $parts[2]
-			. '<?php $_ifcontent = ob_get_contents(); ob_end_flush() ?>'
+			. '<?php $_l->ifcontent = ob_get_flush() ?>'
 			. $parts[3];
-		return 'rtrim($_ifcontent) === "" ? ob_end_clean() : ob_end_flush()';
+		return 'if (rtrim($_l->ifcontent) === "") ob_end_clean(); else echo ob_get_clean()';
 	}
 
 
@@ -194,7 +202,7 @@ class CoreMacros extends MacroSet
 			return $writer->write('echo %modify($template->translate(%node.args))');
 
 		} else {
-			return 'ob_start()';
+			return 'ob_start(function () {})';
 		}
 	}
 
@@ -204,13 +212,13 @@ class CoreMacros extends MacroSet
 	 */
 	public function macroInclude(MacroNode $node, PhpWriter $writer)
 	{
-		$code = $writer->write('Nette\Latte\Macros\CoreMacros::includeTemplate(%node.word, %node.array? + $template->getParameters(), $_l->templates[%var])',
+		$code = $writer->write('$_b->templates[%var]->renderChildTemplate(%node.word, %node.array? + $template->getParameters())',
 			$this->getCompiler()->getTemplateId());
 
 		if ($node->modifiers) {
-			return $writer->write('echo %modify(%raw->__toString(TRUE))', $code);
+			return $writer->write('ob_start(function () {}); %raw; echo %modify(ob_get_clean())', $code);
 		} else {
-			return $code . '->render()';
+			return $code;
 		}
 	}
 
@@ -220,7 +228,10 @@ class CoreMacros extends MacroSet
 	 */
 	public function macroUse(MacroNode $node, PhpWriter $writer)
 	{
-		Nette\Utils\Callback::invoke(array($node->tokenizer->fetchWord(), 'install'), $this->getCompiler())
+		if ($node->modifiers) {
+			trigger_error("Modifiers are not allowed in {{$node->name}}", E_USER_WARNING);
+		}
+		call_user_func(Latte\Helpers::checkCallback(array($node->tokenizer->fetchWord(), 'install')), $this->getCompiler())
 			->initialize();
 	}
 
@@ -235,7 +246,7 @@ class CoreMacros extends MacroSet
 			throw new CompileException("Invalid capture block variable '$variable'");
 		}
 		$node->data->variable = $variable;
-		return 'ob_start()';
+		return 'ob_start(function () {})';
 	}
 
 
@@ -253,8 +264,11 @@ class CoreMacros extends MacroSet
 	 */
 	public function macroEndForeach(MacroNode $node, PhpWriter $writer)
 	{
+		if ($node->modifiers && $node->modifiers !== '|noiterator') {
+			trigger_error('Only modifier |noiterator is allowed here.', E_USER_WARNING);
+		}
 		if ($node->modifiers !== '|noiterator' && preg_match('#\W(\$iterator|include|require|get_defined_vars)\W#', $this->getCompiler()->expandTokens($node->content))) {
-			$node->openingCode = '<?php $iterations = 0; foreach ($iterator = $_l->its[] = new Nette\Iterators\CachingIterator('
+			$node->openingCode = '<?php $iterations = 0; foreach ($iterator = $_l->its[] = new Latte\Runtime\CachingIterator('
 			. preg_replace('#(.*)\s+as\s+#i', '$1) as ', $writer->formatArgs(), 1) . ') { ?>';
 			$node->closingCode = '<?php $iterations++; } array_pop($_l->its); $iterator = end($_l->its) ?>';
 		} else {
@@ -270,6 +284,9 @@ class CoreMacros extends MacroSet
 	 */
 	public function macroBreakContinueIf(MacroNode $node, PhpWriter $writer)
 	{
+		if ($node->modifiers) {
+			trigger_error("Modifiers are not allowed in {{$node->name}}", E_USER_WARNING);
+		}
 		$cmd = str_replace('If', '', $node->name);
 		if ($node->parentNode && $node->parentNode->prefix === $node::PREFIX_NONE) {
 			return $writer->write("if (%node.args) { echo \"</{$node->parentNode->htmlNode->name}>\\n\"; $cmd; }");
@@ -283,7 +300,10 @@ class CoreMacros extends MacroSet
 	 */
 	public function macroClass(MacroNode $node, PhpWriter $writer)
 	{
-		return $writer->write('if ($_l->tmp = array_filter(%node.array)) echo \' class="\' . %escape(implode(" ", array_unique($_l->tmp))) . \'"\'');
+		if (isset($node->htmlNode->attrs['class'])) {
+			throw new CompileException('It is not possible to combine class with n:class.');
+		}
+		return $writer->write('if ($_l->tmp = array_filter(%node.array)) echo \' class="\', %escape(implode(" ", array_unique($_l->tmp))), \'"\'');
 	}
 
 
@@ -292,18 +312,7 @@ class CoreMacros extends MacroSet
 	 */
 	public function macroAttr(MacroNode $node, PhpWriter $writer)
 	{
-		return $writer->write('echo Nette\Utils\Html::el(NULL, %node.array)->attributes()');
-	}
-
-
-	/**
-	 * {attr ...}
-	 * @deprecated
-	 */
-	public function macroOldAttr(MacroNode $node)
-	{
-		trigger_error('Macro {attr} is deprecated; use n:attr="..." instead.', E_USER_DEPRECATED);
-		return Nette\Utils\Strings::replace($node->args . ' ', '#\)\s+#', ')->');
+		return $writer->write('echo Latte\Runtime\Filters::htmlAttributes(%node.array)');
 	}
 
 
@@ -312,9 +321,14 @@ class CoreMacros extends MacroSet
 	 */
 	public function macroDump(MacroNode $node, PhpWriter $writer)
 	{
+		if ($node->modifiers) {
+			trigger_error("Modifiers are not allowed in {{$node->name}}", E_USER_WARNING);
+		}
 		$args = $writer->formatArgs();
-		return 'Nette\Diagnostics\Debugger::barDump(' . ($node->args ? "array(" . $writer->write('%var', $args) . " => $args)" : 'get_defined_vars()')
-			. ', "Template " . str_replace(dirname(dirname($template->getFile())), "\xE2\x80\xA6", $template->getFile()))';
+		return $writer->write(
+			'Tracy\Debugger::barDump(' . ($args ? "($args)" : 'get_defined_vars()'). ', %var)',
+			$args ?: 'variables'
+		);
 	}
 
 
@@ -323,6 +337,9 @@ class CoreMacros extends MacroSet
 	 */
 	public function macroDebugbreak(MacroNode $node, PhpWriter $writer)
 	{
+		if ($node->modifiers) {
+			trigger_error("Modifiers are not allowed in {{$node->name}}", E_USER_WARNING);
+		}
 		return $writer->write(($node->args == NULL ? '' : 'if (!(%node.args)); else')
 			. 'if (function_exists("debugbreak")) debugbreak(); elseif (function_exists("xdebug_break")) xdebug_break()');
 	}
@@ -334,11 +351,11 @@ class CoreMacros extends MacroSet
 	 */
 	public function macroVar(MacroNode $node, PhpWriter $writer)
 	{
+		if ($node->modifiers) {
+			trigger_error("Modifiers are not allowed in {{$node->name}}", E_USER_WARNING);
+		}
 		if ($node->args === '' && $node->parentNode && $node->parentNode->name === 'switch') {
 			return '} else {';
-
-		} elseif ($node->name === 'assign') {
-			trigger_error('Macro {assign} is deprecated; use {var} instead.', E_USER_DEPRECATED);
 		}
 
 		$var = TRUE;
@@ -358,6 +375,9 @@ class CoreMacros extends MacroSet
 				$var = FALSE;
 
 			} elseif ($tokens->isCurrent(',') && $tokens->depth === 0) {
+				if ($var === NULL) {
+					$res->append($node->name === 'default' ? '=>NULL' : '=NULL');
+				}
 				$res->append($node->name === 'default' ? ',' : ';');
 				$var = TRUE;
 
@@ -368,6 +388,9 @@ class CoreMacros extends MacroSet
 				$res->append($tokens->currentToken());
 			}
 		}
+		if ($var === NULL) {
+			$res->append($node->name === 'default' ? '=>NULL' : '=NULL');
+		}
 		$out = $writer->quoteFilter($res)->joinAll();
 		return $node->name === 'default' ? "extract(array($out), EXTR_SKIP)" : $out;
 	}
@@ -379,65 +402,58 @@ class CoreMacros extends MacroSet
 	 */
 	public function macroExpr(MacroNode $node, PhpWriter $writer)
 	{
-		return $writer->write(($node->name === '?' ? '' : 'echo ') . '%modify(%node.args)');
-	}
-
-
-	/********************* run-time helpers ****************d*g**/
-
-
-	/**
-	 * Includes subtemplate.
-	 * @param  mixed      included file name or template
-	 * @param  array      parameters
-	 * @param  Nette\Templating\ITemplate  current template
-	 * @return Nette\Templating\Template
-	 */
-	public static function includeTemplate($destination, array $params, Nette\Templating\ITemplate $template)
-	{
-		if ($destination instanceof Nette\Templating\ITemplate) {
-			$tpl = $destination;
-
-		} elseif ($destination == NULL) { // intentionally ==
-			throw new Nette\InvalidArgumentException("Template file name was not specified.");
-
-		} elseif ($template instanceof Nette\Templating\IFileTemplate) {
-			if (!preg_match('#/|\\\\|[a-z]:#iA', $destination)) {
-				$destination = dirname($template->getFile()) . '/' . $destination;
-			}
-			$tpl = clone $template;
-			$tpl->setFile($destination);
-
-		} else {
-			throw new Nette\NotSupportedException('Macro {include "filename"} is supported only with Nette\Templating\IFileTemplate.');
-		}
-
-		$tpl->setParameters($params); // interface?
-		return $tpl;
+		return $writer->write(($node->name === '=' ? 'echo ' : '') . '%modify(%node.args)');
 	}
 
 
 	/**
-	 * Initializes local & global storage in template.
-	 * @return \stdClass
+	 * {contentType ...}
 	 */
-	public static function initRuntime(Nette\Templating\ITemplate $template, $templateId)
+	public function macroContentType(MacroNode $node, PhpWriter $writer)
 	{
-		// local storage
-		if (isset($template->_l)) {
-			$local = $template->_l;
-			unset($template->_l);
+		if ($node->modifiers) {
+			trigger_error("Modifiers are not allowed in {{$node->name}}", E_USER_WARNING);
+		}
+		if (strpos($node->args, 'xhtml') !== FALSE) {
+			$this->getCompiler()->setContentType(Latte\Compiler::CONTENT_XHTML);
+
+		} elseif (strpos($node->args, 'html') !== FALSE) {
+			$this->getCompiler()->setContentType(Latte\Compiler::CONTENT_HTML);
+
+		} elseif (strpos($node->args, 'xml') !== FALSE) {
+			$this->getCompiler()->setContentType(Latte\Compiler::CONTENT_XML);
+
+		} elseif (strpos($node->args, 'javascript') !== FALSE) {
+			$this->getCompiler()->setContentType(Latte\Compiler::CONTENT_JS);
+
+		} elseif (strpos($node->args, 'css') !== FALSE) {
+			$this->getCompiler()->setContentType(Latte\Compiler::CONTENT_CSS);
+
+		} elseif (strpos($node->args, 'calendar') !== FALSE) {
+			$this->getCompiler()->setContentType(Latte\Compiler::CONTENT_ICAL);
+
 		} else {
-			$local = new \stdClass;
-		}
-		$local->templates[$templateId] = $template;
-
-		// global storage
-		if (!isset($template->_g)) {
-			$template->_g = new \stdClass;
+			$this->getCompiler()->setContentType(Latte\Compiler::CONTENT_TEXT);
 		}
 
-		return array($local, $template->_g);
+		// temporary solution
+		if (strpos($node->args, '/')) {
+			return $writer->write('header(%var)', "Content-Type: $node->args");
+		}
+	}
+
+
+	/**
+	 * {status ...}
+	 */
+	public function macroStatus(MacroNode $node, PhpWriter $writer)
+	{
+		if ($node->modifiers) {
+			trigger_error("Modifiers are not allowed in {{$node->name}}", E_USER_WARNING);
+		}
+		return $writer->write((substr($node->args, -1) === '?' ? 'if (!headers_sent()) ' : '') .
+			'header((isset($_SERVER["SERVER_PROTOCOL"]) ? $_SERVER["SERVER_PROTOCOL"] : "HTTP/1.1") . " " . %0.var, TRUE, %0.var)', (int) $node->args
+		);
 	}
 
 }
