@@ -1,25 +1,20 @@
 <?php
 
 /**
- * This file is part of the Nette Framework (http://nette.org)
- * Copyright (c) 2004 David Grudl (http://davidgrudl.com)
+ * This file is part of the Latte (https://latte.nette.org)
+ * Copyright (c) 2008 David Grudl (https://davidgrudl.com)
  */
 
-namespace Nette\Latte;
-
-use Nette,
-	Nette\Utils\Strings;
+namespace Latte;
 
 
 /**
  * Latte parser.
- *
- * @author     David Grudl
  */
-class Parser extends Nette\Object
+class Parser extends Object
 {
 	/** @internal regular expression for single & double quoted PHP string */
-	const RE_STRING = '\'(?:\\\\.|[^\'\\\\])*\'|"(?:\\\\.|[^"\\\\])*"';
+	const RE_STRING = '\'(?:\\\\.|[^\'\\\\])*+\'|"(?:\\\\.|[^"\\\\])*+"';
 
 	/** @internal special HTML attribute prefix */
 	const N_PREFIX = 'n:';
@@ -28,7 +23,7 @@ class Parser extends Nette\Object
 	public $defaultSyntax = 'latte';
 
 	/** @var bool */
-	public $shortNoEscape = TRUE;
+	public $shortNoEscape = FALSE;
 
 	/** @var array */
 	public $syntaxes = array(
@@ -39,8 +34,8 @@ class Parser extends Nette\Object
 		'off' => array('[^\x00-\xFF]', ''),
 	);
 
-	/** @var string */
-	private $macroRe;
+	/** @var string[] */
+	private $delimiters;
 
 	/** @var string source template */
 	private $input;
@@ -60,6 +55,9 @@ class Parser extends Nette\Object
 	/** @var string used by filter() */
 	private $syntaxEndTag;
 
+	/** @var int */
+	private $syntaxEndLevel = 0;
+
 	/** @var bool */
 	private $xmlMode;
 
@@ -69,7 +67,8 @@ class Parser extends Nette\Object
 		CONTEXT_HTML_TAG = 'htmlTag',
 		CONTEXT_HTML_ATTRIBUTE = 'htmlAttribute',
 		CONTEXT_RAW = 'raw',
-		CONTEXT_HTML_COMMENT = 'htmlComment';
+		CONTEXT_HTML_COMMENT = 'htmlComment',
+		CONTEXT_MACRO = 'macro';
 
 
 	/**
@@ -82,33 +81,32 @@ class Parser extends Nette\Object
 		if (substr($input, 0, 3) === "\xEF\xBB\xBF") { // BOM
 			$input = substr($input, 3);
 		}
-		if (!Strings::checkEncoding($input)) {
-			throw new Nette\InvalidArgumentException('Template is not valid UTF-8 stream.');
-		}
-		$input = str_replace("\r\n", "\n", $input);
-		$this->input = $input;
-		$this->output = array();
+
+		$this->input = $input = str_replace("\r\n", "\n", $input);
 		$this->offset = 0;
+		$this->output = array();
+
+		if (!preg_match('##u', $input)) {
+			preg_match('#(?:[\x00-\x7F]|[\xC0-\xDF][\x80-\xBF]|[\xE0-\xEF][\x80-\xBF]{2}|[\xF0-\xF7][\x80-\xBF]{3})*+#A', $input, $m);
+			$this->offset = strlen($m[0]) + 1;
+			throw new \InvalidArgumentException('Template is not valid UTF-8 stream.');
+		}
 
 		$this->setSyntax($this->defaultSyntax);
 		$this->setContext(self::CONTEXT_HTML_TEXT);
 		$this->lastHtmlTag = $this->syntaxEndTag = NULL;
 
+		$tokenCount = 0;
 		while ($this->offset < strlen($input)) {
-			$matches = $this->{"context".$this->context[0]}();
-
-			if (!$matches) { // EOF
+			if ($this->{'context' . $this->context[0]}() === FALSE) {
 				break;
-
-			} elseif (!empty($matches['comment'])) { // {* *}
-				$this->addToken(Token::COMMENT, $matches[0]);
-
-			} elseif (!empty($matches['macro'])) { // {macro}
-				$token = $this->addToken(Token::MACRO_TAG, $matches[0]);
-				list($token->name, $token->value, $token->modifiers, $token->empty) = $this->parseMacroTag($matches['macro']);
 			}
-
-			$this->filter();
+			while ($tokenCount < count($this->output)) {
+				$this->filter($this->output[$tokenCount++]);
+			}
+		}
+		if ($this->context[0] === self::CONTEXT_MACRO) {
+			throw new CompileException('Malformed macro');
 		}
 
 		if ($this->offset < strlen($input)) {
@@ -126,7 +124,7 @@ class Parser extends Nette\Object
 		$matches = $this->match('~
 			(?:(?<=\n|^)[ \t]*)?<(?P<closing>/?)(?P<tag>[a-z0-9:]+)|  ##  begin of HTML tag <tag </tag - ignores <!DOCTYPE
 			<(?P<htmlcomment>!--(?!>))|     ##  begin of HTML comment <!--, but not <!-->
-			'.$this->macroRe.'              ##  macro tag
+			(?P<macro>' . $this->delimiters[0] . ')
 		~xsi');
 
 		if (!empty($matches['htmlcomment'])) { // <!--
@@ -139,8 +137,10 @@ class Parser extends Nette\Object
 			$token->closing = (bool) $matches['closing'];
 			$this->lastHtmlTag = $matches['closing'] . strtolower($matches['tag']);
 			$this->setContext(self::CONTEXT_HTML_TAG);
+
+		} else {
+			return $this->processMacro($matches);
 		}
-		return $matches;
 	}
 
 
@@ -150,8 +150,8 @@ class Parser extends Nette\Object
 	private function contextCData()
 	{
 		$matches = $this->match('~
-			</(?P<tag>'.$this->lastHtmlTag.')(?![a-z0-9:])| ##  end HTML tag </tag
-			'.$this->macroRe.'              ##  macro tag
+			</(?P<tag>' . $this->lastHtmlTag . ')(?![a-z0-9:])| ##  end HTML tag </tag
+			(?P<macro>' . $this->delimiters[0] . ')
 		~xsi');
 
 		if (!empty($matches['tag'])) { // </tag
@@ -160,8 +160,9 @@ class Parser extends Nette\Object
 			$token->closing = TRUE;
 			$this->lastHtmlTag = '/' . $this->lastHtmlTag;
 			$this->setContext(self::CONTEXT_HTML_TAG);
+		} else {
+			return $this->processMacro($matches);
 		}
-		return $matches;
 	}
 
 
@@ -172,7 +173,7 @@ class Parser extends Nette\Object
 	{
 		$matches = $this->match('~
 			(?P<end>\ ?/?>)([ \t]*\n)?|  ##  end of HTML tag
-			'.$this->macroRe.'|          ##  macro tag
+			(?P<macro>' . $this->delimiters[0] . ')|
 			\s*(?P<attr>[^\s/>={]+)(?:\s*=\s*(?P<value>["\']|[^\s/>{]+))? ## beginning of HTML attribute
 		~xsi');
 
@@ -186,7 +187,7 @@ class Parser extends Nette\Object
 			$token->value = isset($matches['value']) ? $matches['value'] : '';
 
 			if ($token->value === '"' || $token->value === "'") { // attribute = "'
-				if (Strings::startsWith($token->name, self::N_PREFIX)) {
+				if (strncmp($token->name, self::N_PREFIX, strlen(self::N_PREFIX)) === 0) {
 					$token->value = '';
 					if ($m = $this->match('~(.*?)' . $matches['value'] . '~xsi')) {
 						$token->value = $m[1];
@@ -196,8 +197,9 @@ class Parser extends Nette\Object
 					$this->setContext(self::CONTEXT_HTML_ATTRIBUTE, $matches['value']);
 				}
 			}
+		} else {
+			return $this->processMacro($matches);
 		}
-		return $matches;
 	}
 
 
@@ -207,15 +209,16 @@ class Parser extends Nette\Object
 	private function contextHtmlAttribute()
 	{
 		$matches = $this->match('~
-			(?P<quote>'.$this->context[1].')|  ##  end of HTML attribute
-			'.$this->macroRe.'                 ##  macro tag
+			(?P<quote>' . $this->context[1] . ')|  ##  end of HTML attribute
+			(?P<macro>' . $this->delimiters[0] . ')
 		~xsi');
 
 		if (!empty($matches['quote'])) { // (attribute end) '"
 			$this->addToken(Token::TEXT, $matches[0]);
 			$this->setContext(self::CONTEXT_HTML_TAG);
+		} else {
+			return $this->processMacro($matches);
 		}
-		return $matches;
 	}
 
 
@@ -226,14 +229,15 @@ class Parser extends Nette\Object
 	{
 		$matches = $this->match('~
 			(?P<htmlcomment>-->)|   ##  end of HTML comment
-			'.$this->macroRe.'      ##  macro tag
+			(?P<macro>' . $this->delimiters[0] . ')
 		~xsi');
 
 		if (!empty($matches['htmlcomment'])) { // -->
 			$this->addToken(Token::HTML_TAG_END, $matches[0]);
 			$this->setContext(self::CONTEXT_HTML_TEXT);
+		} else {
+			return $this->processMacro($matches);
 		}
-		return $matches;
 	}
 
 
@@ -243,9 +247,50 @@ class Parser extends Nette\Object
 	private function contextRaw()
 	{
 		$matches = $this->match('~
-			'.$this->macroRe.'     ##  macro tag
+			(?P<macro>' . $this->delimiters[0] . ')
 		~xsi');
-		return $matches;
+		return $this->processMacro($matches);
+	}
+
+
+	/**
+	 * Handles CONTEXT_MACRO.
+	 */
+	private function contextMacro()
+	{
+		$matches = $this->match('~
+			(?P<comment>\\*.*?\\*' . $this->delimiters[1] . '\n{0,2})|
+			(?P<macro>(?>
+				' . self::RE_STRING . '|
+				\{(?>' . self::RE_STRING . '|[^\'"{}])*+\}|
+				[^\'"{}]
+			)+?)
+			' . $this->delimiters[1] . '
+			(?P<rmargin>[ \t]*(?=\n))?
+		~xsiA');
+
+		if (!empty($matches['macro'])) {
+			$token = $this->addToken(Token::MACRO_TAG, $this->context[1][1] . $matches[0]);
+			list($token->name, $token->value, $token->modifiers, $token->empty) = $this->parseMacroTag($matches['macro']);
+			$this->context = $this->context[1][0];
+
+		} elseif (!empty($matches['comment'])) {
+			$this->addToken(Token::COMMENT, $this->context[1][1] . $matches[0]);
+			$this->context = $this->context[1][0];
+
+		} else {
+			throw new CompileException('Malformed macro');
+		}
+	}
+
+
+	private function processMacro($matches)
+	{
+		if (!empty($matches['macro'])) { // {macro} or {* *}
+			$this->setContext(self::CONTEXT_MACRO, array($this->context, $matches['macro']));
+		} else {
+			return FALSE;
+		}
 	}
 
 
@@ -256,20 +301,43 @@ class Parser extends Nette\Object
 	 */
 	private function match($re)
 	{
-		if ($matches = Strings::match($this->input, $re, PREG_OFFSET_CAPTURE, $this->offset)) {
-			$value = substr($this->input, $this->offset, $matches[0][1] - $this->offset);
-			if ($value !== '') {
-				$this->addToken(Token::TEXT, $value);
-			}
-			$this->offset = $matches[0][1] + strlen($matches[0][0]);
-			foreach ($matches as $k => $v) {
-				$matches[$k] = $v[0];
+		if (!preg_match($re, $this->input, $matches, PREG_OFFSET_CAPTURE, $this->offset)) {
+			if (preg_last_error()) {
+				throw new RegexpException(NULL, preg_last_error());
 			}
+			return array();
+		}
+
+		$value = substr($this->input, $this->offset, $matches[0][1] - $this->offset);
+		if ($value !== '') {
+			$this->addToken(Token::TEXT, $value);
+		}
+		$this->offset = $matches[0][1] + strlen($matches[0][0]);
+		foreach ($matches as $k => $v) {
+			$matches[$k] = $v[0];
 		}
 		return $matches;
 	}
 
 
+	/**
+	 * @return self
+	 */
+	public function setContentType($type)
+	{
+		if (strpos($type, 'html') !== FALSE) {
+			$this->xmlMode = FALSE;
+			$this->setContext(self::CONTEXT_HTML_TEXT);
+		} elseif (strpos($type, 'xml') !== FALSE) {
+			$this->xmlMode = TRUE;
+			$this->setContext(self::CONTEXT_HTML_TEXT);
+		} else {
+			$this->setContext(self::CONTEXT_RAW);
+		}
+		return $this;
+	}
+
+
 	/**
 	 * @return self
 	 */
@@ -291,7 +359,7 @@ class Parser extends Nette\Object
 		if (isset($this->syntaxes[$type])) {
 			$this->setDelimiters($this->syntaxes[$type][0], $this->syntaxes[$type][1]);
 		} else {
-			throw new Nette\InvalidArgumentException("Unknown syntax '$type'");
+			throw new \InvalidArgumentException("Unknown syntax '$type'");
 		}
 		return $this;
 	}
@@ -305,17 +373,7 @@ class Parser extends Nette\Object
 	 */
 	public function setDelimiters($left, $right)
 	{
-		$this->macroRe = '
-			(?P<comment>' . $left . '\\*.*?\\*' . $right . '\n{0,2})|
-			' . $left . '
-				(?P<macro>(?:
-					' . self::RE_STRING . '|
-					\{(?:' . self::RE_STRING . '|[^\'"{}])*+\}|
-					[^\'"{}]
-				)+?)
-			' . $right . '
-			(?P<rmargin>[ \t]*(?=\n))?
-		';
+		$this->delimiters = array($left, $right);
 		return $this;
 	}
 
@@ -324,19 +382,21 @@ class Parser extends Nette\Object
 	 * Parses macro tag to name, arguments a modifiers parts.
 	 * @param  string {name arguments | modifiers}
 	 * @return array
+	 * @internal
 	 */
 	public function parseMacroTag($tag)
 	{
-		$match = Strings::match($tag, '~^
+		if (!preg_match('~^
 			(
 				(?P<name>\?|/?[a-z]\w*+(?:[.:]\w+)*+(?!::|\(|\\\\))|   ## ?, name, /name, but not function( or class:: or namespace\
 				(?P<noescape>!?)(?P<shortname>/?[=\~#%^&_]?)      ## !expression, !=expression, ...
-			)(?P<args>.*?)
-			(?P<modifiers>\|[a-z](?:'.Parser::RE_STRING.'|[^\'"])*(?<!/))?
+			)(?P<args>(?:' . self::RE_STRING . '|[^\'"])*?)
+			(?P<modifiers>(?<!\|)\|[a-z](?:' . self::RE_STRING . '|[^\'"/]|/(?=.))*+)?
 			(?P<empty>/?\z)
-		()\z~isx');
-
-		if (!$match) {
+		()\z~isx', $tag, $match)) {
+			if (preg_last_error()) {
+				throw new RegexpException(NULL, preg_last_error());
+			}
 			return FALSE;
 		}
 		if ($match['name'] === '') {
@@ -362,18 +422,19 @@ class Parser extends Nette\Object
 	}
 
 
-	private function getLine()
+	public function getLine()
 	{
-		return substr_count($this->input, "\n", 0, max(1, $this->offset - 1)) + 1;
+		return $this->offset
+			? substr_count(substr($this->input, 0, $this->offset - 1), "\n") + 1
+			: 1;
 	}
 
 
 	/**
 	 * Process low-level macros.
 	 */
-	protected function filter()
+	protected function filter(Token $token)
 	{
-		$token = end($this->output);
 		if ($token->type === Token::MACRO_TAG && $token->name === '/syntax') {
 			$this->setSyntax($this->defaultSyntax);
 			$token->type = Token::COMMENT;
@@ -384,19 +445,18 @@ class Parser extends Nette\Object
 
 		} elseif ($token->type === Token::HTML_ATTRIBUTE && $token->name === 'n:syntax') {
 			$this->setSyntax($token->value);
-			$this->syntaxEndTag = '/' . $this->lastHtmlTag;
+			$this->syntaxEndTag = $this->lastHtmlTag;
+			$this->syntaxEndLevel = 1;
 			$token->type = Token::COMMENT;
 
-		} elseif ($token->type === Token::HTML_TAG_END && $this->lastHtmlTag === $this->syntaxEndTag) {
+		} elseif ($token->type === Token::HTML_TAG_BEGIN && $this->lastHtmlTag === $this->syntaxEndTag) {
+			$this->syntaxEndLevel++;
+
+		} elseif ($token->type === Token::HTML_TAG_END && $this->lastHtmlTag === ('/' . $this->syntaxEndTag) && --$this->syntaxEndLevel === 0) {
 			$this->setSyntax($this->defaultSyntax);
 
 		} elseif ($token->type === Token::MACRO_TAG && $token->name === 'contentType') {
-			if (preg_match('#html|xml#', $token->value, $m)) {
-				$this->xmlMode = $m[0] === 'xml';
-				$this->setContext(self::CONTEXT_HTML_TEXT);
-			} else {
-				$this->setContext(self::CONTEXT_RAW);
-			}
+			$this->setContentType($token->value);
 		}
 	}
 
