@@ -1,27 +1,21 @@
 <?php
 
 /**
- * This file is part of the Nette Framework (http://nette.org)
- * Copyright (c) 2004 David Grudl (http://davidgrudl.com)
+ * This file is part of the Nette Framework (https://nette.org)
+ * Copyright (c) 2004 David Grudl (https://davidgrudl.com)
  */
 
 namespace Nette\Mail;
 
-use Nette,
-	Nette\Utils\Strings;
+use Nette;
+use Nette\Utils\Strings;
 
 
 /**
  * Mail provides functionality to compose and send both text and MIME-compliant multipart email messages.
  *
- * @author     David Grudl
- *
- * @property   array $from
  * @property   string $subject
- * @property   string $returnPath
- * @property   int $priority
  * @property   mixed $htmlBody
- * @property   IMailer $mailer
  */
 class Message extends MimePart
 {
@@ -30,18 +24,12 @@ class Message extends MimePart
 		NORMAL = 3,
 		LOW = 5;
 
-	/** @deprecated */
-	public static $defaultMailer = 'Nette\Mail\SendmailMailer';
-
 	/** @var array */
 	public static $defaultHeaders = array(
 		'MIME-Version' => '1.0',
 		'X-Mailer' => 'Nette Framework',
 	);
 
-	/** @var IMailer */
-	private $mailer;
-
 	/** @var array */
 	private $attachments = array();
 
@@ -220,27 +208,34 @@ class Message extends MimePart
 
 	/**
 	 * Sets HTML body.
-	 * @param  string|Nette\Templating\ITemplate
-	 * @param  mixed base-path or FALSE to disable parsing
+	 * @param  string
+	 * @param  mixed base-path
 	 * @return self
 	 */
 	public function setHtmlBody($html, $basePath = NULL)
 	{
-		if ($html instanceof Nette\Templating\ITemplate) {
-			$html->mail = $this;
-			if ($basePath === NULL && $html instanceof Nette\Templating\IFileTemplate) {
-				$basePath = dirname($html->getFile());
-			}
-			$html = $html->__toString(TRUE);
+		if ($basePath === NULL && ($html instanceof Nette\Templating\IFileTemplate || $html instanceof Nette\Application\UI\ITemplate)) {
+			$basePath = dirname($html->getFile());
+			$bc = TRUE;
 		}
+		$html = (string) $html;
 
-		if ($basePath !== FALSE) {
+		if ($basePath) {
 			$cids = array();
 			$matches = Strings::matchAll(
 				$html,
-				'#(src\s*=\s*|background\s*=\s*|url\()(["\']?)(?![a-z]+:|[/\\#])([^"\')\s]+)#i',
+				'#
+					(<img[^<>]*\s src\s*=\s*
+					|<body[^<>]*\s background\s*=\s*
+					|<[^<>]+\s style\s*=\s* ["\'][^"\'>]+[:\s] url\(
+					|<style[^>]*>[^<]+ [:\s] url\()
+					(["\']?)(?![a-z]+:|[/\\#])([^"\'>)\s]+)
+				#ix',
 				PREG_OFFSET_CAPTURE
 			);
+			if ($matches && isset($bc)) {
+				trigger_error(__METHOD__ . '() missing second argument with image base path.', E_USER_WARNING);
+			}
 			foreach (array_reverse($matches) as $m) {
 				$file = rtrim($basePath, '/\\') . '/' . urldecode($m[3][0]);
 				if (!isset($cids[$file])) {
@@ -252,15 +247,20 @@ class Message extends MimePart
 				);
 			}
 		}
-		$this->html = $html;
 
-		if ($this->getSubject() == NULL && $matches = Strings::match($html, '#<title>(.+?)</title>#is')) { // intentionally ==
-			$this->setSubject(html_entity_decode($matches[1], ENT_QUOTES, 'UTF-8'));
+		if ($this->getSubject() == NULL) { // intentionally ==
+			$html = Strings::replace($html, '#<title>(.+?)</title>#is', function ($m) use (& $title) {
+				$title = $m[1];
+			});
+			$this->setSubject(html_entity_decode($title, ENT_QUOTES, 'UTF-8'));
 		}
 
+		$this->html = ltrim(str_replace("\r", '', $html), "\n");
+
 		if ($this->getBody() == NULL && $html != NULL) { // intentionally ==
 			$this->setBody($this->buildText($html));
 		}
+
 		return $this;
 	}
 
@@ -302,6 +302,16 @@ class Message extends MimePart
 	}
 
 
+	/**
+	 * Gets all email attachments.
+	 * @return MimePart[]
+	 */
+	public function getAttachments()
+	{
+		return $this->attachments;
+	}
+
+
 	/**
 	 * Creates file MIME part.
 	 * @return MimePart
@@ -318,7 +328,7 @@ class Message extends MimePart
 			$content = (string) $content;
 		}
 		$part->setBody($content);
-		$part->setContentType($contentType ? $contentType : Nette\Utils\MimeTypeDetector::fromString($content));
+		$part->setContentType($contentType ? $contentType : finfo_buffer(finfo_open(FILEINFO_MIME_TYPE), $content));
 		$part->setEncoding(preg_match('#(multipart|message)/#A', $contentType) ? self::ENCODING_8BIT : self::ENCODING_BASE64);
 		$part->setHeader('Content-Disposition', $disposition . '; filename="' . Strings::fixEncoding(basename($file)) . '"');
 		return $part;
@@ -328,40 +338,6 @@ class Message extends MimePart
 	/********************* building and sending ****************d*g**/
 
 
-	/**
-	 * @deprecated
-	 */
-	public function send()
-	{
-		trigger_error(__METHOD__ . '() is deprecated; use IMailer::send() instead.', E_USER_DEPRECATED);
-		$this->getMailer()->send($this);
-	}
-
-
-	/**
-	 * @deprecated
-	 */
-	public function setMailer(IMailer $mailer)
-	{
-		//trigger_error(__METHOD__ . '() is deprecated.', E_USER_DEPRECATED);
-		$this->mailer = $mailer;
-		return $this;
-	}
-
-
-	/**
-	 * @deprecated
-	 */
-	public function getMailer()
-	{
-		trigger_error(__METHOD__ . '() is deprecated.', E_USER_DEPRECATED);
-		if ($this->mailer === NULL) {
-			$this->mailer = is_object(static::$defaultMailer) ? static::$defaultMailer : new static::$defaultMailer;
-		}
-		return $this->mailer;
-	}
-
-
 	/**
 	 * Returns encoded message.
 	 * @return string
@@ -374,7 +350,7 @@ class Message extends MimePart
 
 	/**
 	 * Builds email. Does not modify itself, but returns a new object.
-	 * @return Message
+	 * @return self
 	 */
 	protected function build()
 	{
@@ -402,7 +378,7 @@ class Message extends MimePart
 				}
 			}
 			$alt->setContentType('text/html', 'UTF-8')
-				->setEncoding(preg_match('#\S{990}#', $mail->html)
+				->setEncoding(preg_match('#[^\n]{990}#', $mail->html)
 					? self::ENCODING_QUOTED_PRINTABLE
 					: (preg_match('#[\x80-\xFF]#', $mail->html) ? self::ENCODING_8BIT : self::ENCODING_7BIT))
 				->setBody($mail->html);
@@ -411,7 +387,7 @@ class Message extends MimePart
 		$text = $mail->getBody();
 		$mail->setBody(NULL);
 		$cursor->setContentType('text/plain', 'UTF-8')
-			->setEncoding(preg_match('#\S{990}#', $text)
+			->setEncoding(preg_match('#[^\n]{990}#', $text)
 				? self::ENCODING_QUOTED_PRINTABLE
 				: (preg_match('#[\x80-\xFF]#', $text) ? self::ENCODING_8BIT : self::ENCODING_7BIT))
 			->setBody($text);
@@ -428,7 +404,8 @@ class Message extends MimePart
 	{
 		$text = Strings::replace($html, array(
 			'#<(style|script|head).*</\\1>#Uis' => '',
-			'#<t[dh][ >]#i' => " $0",
+			'#<t[dh][ >]#i' => ' $0',
+			'#<a [^>]*href=(?|"([^"]+)"|\'([^\']+)\')[^>]*>(.*?)</a>#i' =>  '$2 &lt;$1&gt;',
 			'#[\r\n]+#' => ' ',
 			'#<(/?p|/?h\d|li|br|/tr)[ >/]#i' => "\n$0",
 		));
@@ -441,7 +418,7 @@ class Message extends MimePart
 	/** @return string */
 	private function getRandomId()
 	{
-		return '<' . Strings::random() . '@'
+		return '<' . Nette\Utils\Random::generate() . '@'
 			. preg_replace('#[^\w.-]+#', '', isset($_SERVER['HTTP_HOST']) ? $_SERVER['HTTP_HOST'] : php_uname('n'))
 			. '>';
 	}
