@@ -1,20 +1,18 @@
 <?php
 
 /**
- * This file is part of the Nette Framework (http://nette.org)
- * Copyright (c) 2004 David Grudl (http://davidgrudl.com)
+ * This file is part of the Nette Framework (https://nette.org)
+ * Copyright (c) 2004 David Grudl (https://davidgrudl.com)
  */
 
 namespace Nette\Reflection;
 
-use Nette,
-	Nette\Utils\Strings;
+use Nette;
+use Nette\Utils\Strings;
 
 
 /**
  * Annotations support for PHP.
- *
- * @author     David Grudl
  * @Annotation
  */
 class AnnotationsParser
@@ -29,7 +27,7 @@ class AnnotationsParser
 	public static $useReflection;
 
 	/** @var bool */
-	public static $useAnnotationClasses = TRUE;
+	public static $autoRefresh = TRUE;
 
 	/** @var array */
 	public static $inherited = array('description', 'param', 'return');
@@ -63,18 +61,25 @@ class AnnotationsParser
 		if ($r instanceof \ReflectionClass) {
 			$type = $r->getName();
 			$member = 'class';
+			$file = $r->getFileName();
 
 		} elseif ($r instanceof \ReflectionMethod) {
 			$type = $r->getDeclaringClass()->getName();
 			$member = $r->getName();
+			$file = $r->getFileName();
+
+		} elseif ($r instanceof \ReflectionFunction) {
+			$type = NULL;
+			$member = $r->getName();
+			$file = $r->getFileName();
 
 		} else {
 			$type = $r->getDeclaringClass()->getName();
 			$member = '$' . $r->getName();
+			$file = $r->getDeclaringClass()->getFileName();
 		}
 
 		if (!self::$useReflection) { // auto-expire cache
-			$file = $r instanceof \ReflectionClass ? $r->getFileName() : $r->getDeclaringClass()->getFileName(); // will be used later
 			if ($file && isset(self::$timestamps[$file]) && self::$timestamps[$file] !== filemtime($file)) {
 				unset(self::$cache[$type]);
 			}
@@ -93,11 +98,7 @@ class AnnotationsParser
 			$annotations = self::parseComment($r->getDocComment());
 
 		} else {
-			if (!self::$cacheStorage) {
-				// trigger_error('Set a cache storage for annotations parser via Nette\Reflection\AnnotationParser::setCacheStorage().', E_USER_WARNING);
-				self::$cacheStorage = new Nette\Caching\Storages\DevNullStorage;
-			}
-			$outerCache = new Nette\Caching\Cache(self::$cacheStorage, 'Nette.Reflection.Annotations');
+			$outerCache = self::getCache();
 
 			if (self::$cache === NULL) {
 				self::$cache = (array) $outerCache->load('list');
@@ -106,7 +107,7 @@ class AnnotationsParser
 
 			if (!isset(self::$cache[$type]) && $file) {
 				self::$cache['*'][$file] = filemtime($file);
-				foreach (self::parsePhp(file_get_contents($file)) as $class => $info) {
+				foreach (static::parsePhp(file_get_contents($file)) as $class => $info) {
 					foreach ($info as $prop => $comment) {
 						if ($prop !== 'use') {
 							self::$cache[$class][$prop] = self::parseComment($comment);
@@ -152,13 +153,21 @@ class AnnotationsParser
 	{
 		if (empty($name)) {
 			throw new Nette\InvalidArgumentException('Class name must not be empty.');
-		}
 
-		if ($name[0] === '\\') { // already fully qualified
+		} elseif ($name === 'self') {
+			return $reflector->getName();
+
+		} elseif ($name[0] === '\\') { // already fully qualified
 			return ltrim($name, '\\');
 		}
 
-		$parsed = static::parsePhp(file_get_contents($reflector->getFileName()));
+		$filename = $reflector->getFileName();
+		$parsed = static::getCache()->load($filename, function (& $dp) use ($filename) {
+			if (AnnotationsParser::$autoRefresh) {
+				$dp[Nette\Caching\Cache::FILES] = $filename;
+			}
+			return AnnotationsParser::parsePhp(file_get_contents($filename));
+		});
 		$uses = array_change_key_case((array) $tmp = & $parsed[$reflector->getName()]['use']);
 		$parts = explode('\\', $name, 2);
 		$parts[0] = strtolower($parts[0]);
@@ -250,13 +259,7 @@ class AnnotationsParser
 				}
 			}
 
-			$class = $name . 'Annotation';
-			if (self::$useAnnotationClasses && class_exists($class)) {
-				$res[$name][] = new $class(is_array($value) ? $value : array('value' => $value));
-
-			} else {
-				$res[$name][] = is_array($value) ? Nette\ArrayHash::from($value) : $value;
-			}
+			$res[$name][] = is_array($value) ? Nette\Utils\ArrayHash::from($value) : $value;
 		}
 
 		return $res;
@@ -296,6 +299,7 @@ class AnnotationsParser
 					if ($name = self::fetch($tokens, T_STRING)) {
 						$class = $namespace . $name;
 						$classLevel = $level + 1;
+						$res[$class] = array();
 						if ($docComment) {
 							$res[$class]['class'] = $docComment;
 						}
@@ -389,7 +393,19 @@ class AnnotationsParser
 	 */
 	public static function getCacheStorage()
 	{
+		if (!self::$cacheStorage) {
+			self::$cacheStorage = new Nette\Caching\Storages\MemoryStorage();
+		}
 		return self::$cacheStorage;
 	}
 
+
+	/**
+	 * @return Nette\Caching\Cache
+	 */
+	private static function getCache()
+	{
+		return new Nette\Caching\Cache(static::getCacheStorage(), 'Nette.Reflection.Annotations');
+	}
+
 }
