@@ -1,20 +1,18 @@
 <?php
 
 /**
- * This file is part of the Nette Framework (http://nette.org)
- * Copyright (c) 2004 David Grudl (http://davidgrudl.com)
+ * This file is part of the Nette Framework (https://nette.org)
+ * Copyright (c) 2004 David Grudl (https://davidgrudl.com)
  */
 
 namespace Nette\Reflection;
 
-use Nette,
-	Nette\ObjectMixin;
+use Nette;
+use Nette\Utils\ObjectMixin;
 
 
 /**
  * Reports information about a classes variable.
- *
- * @author     David Grudl
  * @property-read ClassType $declaringClass
  * @property-read IAnnotation[][] $annotations
  * @property-read string $description
@@ -100,16 +98,6 @@ class Property extends \ReflectionProperty
 	/********************* Nette\Object behaviour ****************d*g**/
 
 
-	/**
-	 * @deprecated
-	 */
-	public static function getReflection()
-	{
-		trigger_error(__METHOD__ . '() is deprecated.', E_USER_DEPRECATED);
-		return new ClassType(get_called_class());
-	}
-
-
 	public function __call($name, $args)
 	{
 		return ObjectMixin::call($this, $name, $args);
