@@ -1,8 +1,8 @@
 <?php
 
 /**
- * This file is part of the Nette Framework (http://nette.org)
- * Copyright (c) 2004 David Grudl (http://davidgrudl.com)
+ * This file is part of the Nette Framework (https://nette.org)
+ * Copyright (c) 2004 David Grudl (https://davidgrudl.com)
  */
 
 namespace Nette\Utils;
@@ -20,10 +20,8 @@ use Nette;
  *
  * echo $el->startTag(), $el->endTag();
  * </code>
- *
- * @author     David Grudl
  */
-class Html extends Nette\Object implements \ArrayAccess, \Countable, \IteratorAggregate
+class Html extends Nette\Object implements \ArrayAccess, \Countable, \IteratorAggregate, IHtmlString
 {
 	/** @var string  element's name */
 	private $name;
@@ -41,15 +39,18 @@ class Html extends Nette\Object implements \ArrayAccess, \Countable, \IteratorAg
 	public static $xhtml = FALSE;
 
 	/** @var array  empty (void) elements */
-	public static $emptyElements = array('img'=>1,'hr'=>1,'br'=>1,'input'=>1,'meta'=>1,'area'=>1,'embed'=>1,'keygen'=>1,
-		'source'=>1,'base'=>1,'col'=>1,'link'=>1,'param'=>1,'basefont'=>1,'frame'=>1,'isindex'=>1,'wbr'=>1,'command'=>1,'track'=>1);
+	public static $emptyElements = array(
+		'img' => 1, 'hr' => 1, 'br' => 1, 'input' => 1, 'meta' => 1, 'area' => 1, 'embed' => 1, 'keygen' => 1,
+		'source' => 1, 'base' => 1, 'col' => 1, 'link' => 1, 'param' => 1, 'basefont' => 1, 'frame' => 1,
+		'isindex' => 1, 'wbr' => 1, 'command' => 1, 'track' => 1,
+	);
 
 
 	/**
 	 * Static factory.
 	 * @param  string element name (or NULL)
-	 * @param  array|string element's attributes (or textual content)
-	 * @return Html
+	 * @param  array|string element's attributes or plain text content
+	 * @return self
 	 */
 	public static function el($name = NULL, $attrs = NULL)
 	{
@@ -228,9 +229,24 @@ class Html extends Nette\Object implements \ArrayAccess, \Countable, \IteratorAg
 	}
 
 
+	/**
+	 * Setter for data-* attributes. Booleans are converted to 'true' resp. 'false'.
+	 * @return self
+	 */
+	public function data($name, $value = NULL)
+	{
+		if (func_num_args() === 1) {
+			$this->attrs['data'] = $name;
+		} else {
+			$this->attrs["data-$name"] = is_bool($value) ? json_encode($value) : $value;
+		}
+		return $this;
+	}
+
+
 	/**
 	 * Sets element's HTML content.
-	 * @param  string
+	 * @param  string raw HTML string
 	 * @return self
 	 * @throws Nette\InvalidArgumentException
 	 */
@@ -272,7 +288,7 @@ class Html extends Nette\Object implements \ArrayAccess, \Countable, \IteratorAg
 	public function setText($text)
 	{
 		if (!is_array($text) && !$text instanceof self) {
-			$text = htmlspecialchars((string) $text, ENT_NOQUOTES);
+			$text = htmlspecialchars((string) $text, ENT_NOQUOTES, 'UTF-8');
 		}
 		return $this->setHtml($text);
 	}
@@ -290,7 +306,7 @@ class Html extends Nette\Object implements \ArrayAccess, \Countable, \IteratorAg
 
 	/**
 	 * Adds new element's child.
-	 * @param  Html|string child node
+	 * @param  Html|string Html node or raw HTML string
 	 * @return self
 	 */
 	public function add($child)
@@ -302,8 +318,8 @@ class Html extends Nette\Object implements \ArrayAccess, \Countable, \IteratorAg
 	/**
 	 * Creates and adds a new Html child.
 	 * @param  string  elements's name
-	 * @param  array|string element's attributes (or textual content)
-	 * @return Html  created element
+	 * @param  array|string element's attributes or raw HTML string
+	 * @return self  created element
 	 */
 	public function create($name, $attrs = NULL)
 	{
@@ -314,15 +330,15 @@ class Html extends Nette\Object implements \ArrayAccess, \Countable, \IteratorAg
 
 	/**
 	 * Inserts child node.
-	 * @param  int
-	 * @param  Html node
+	 * @param  int|NULL position or NULL for appending
+	 * @param  Html|string Html node or raw HTML string
 	 * @param  bool
 	 * @return self
-	 * @throws \Exception
+	 * @throws Nette\InvalidArgumentException
 	 */
 	public function insert($index, $child, $replace = FALSE)
 	{
-		if ($child instanceof Html || is_scalar($child)) {
+		if ($child instanceof self || is_scalar($child)) {
 			if ($index === NULL) { // append
 				$this->children[] = $child;
 
@@ -340,8 +356,8 @@ class Html extends Nette\Object implements \ArrayAccess, \Countable, \IteratorAg
 
 	/**
 	 * Inserts (replaces) child node (\ArrayAccess implementation).
-	 * @param  int
-	 * @param  Html node
+	 * @param  int|NULL position or NULL for appending
+	 * @param  Html|string Html node or raw HTML string
 	 * @return void
 	 */
 	public function offsetSet($index, $child)
@@ -352,8 +368,8 @@ class Html extends Nette\Object implements \ArrayAccess, \Countable, \IteratorAg
 
 	/**
 	 * Returns child node (\ArrayAccess implementation).
-	 * @param  int index
-	 * @return mixed
+	 * @param  int
+	 * @return self|string
 	 */
 	public function offsetGet($index)
 	{
@@ -363,7 +379,7 @@ class Html extends Nette\Object implements \ArrayAccess, \Countable, \IteratorAg
 
 	/**
 	 * Exists child node? (\ArrayAccess implementation).
-	 * @param  int index
+	 * @param  int
 	 * @return bool
 	 */
 	public function offsetExists($index)
@@ -374,7 +390,7 @@ class Html extends Nette\Object implements \ArrayAccess, \Countable, \IteratorAg
 
 	/**
 	 * Removes child node (\ArrayAccess implementation).
-	 * @param  int index
+	 * @param  int
 	 * @return void
 	 */
 	public function offsetUnset($index)
@@ -386,7 +402,7 @@ class Html extends Nette\Object implements \ArrayAccess, \Countable, \IteratorAg
 
 
 	/**
-	 * Required by the \Countable interface.
+	 * Returns children count.
 	 * @return int
 	 */
 	public function count()
@@ -396,7 +412,7 @@ class Html extends Nette\Object implements \ArrayAccess, \Countable, \IteratorAg
 
 
 	/**
-	 * Removed all children.
+	 * Removes all children.
 	 * @return void
 	 */
 	public function removeChildren()
@@ -406,25 +422,17 @@ class Html extends Nette\Object implements \ArrayAccess, \Countable, \IteratorAg
 
 
 	/**
-	 * Iterates over a elements.
-	 * @param  bool    recursive?
-	 * @param  string  class types filter
-	 * @return \RecursiveIterator
+	 * Iterates over elements.
+	 * @return \ArrayIterator
 	 */
-	public function getIterator($deep = FALSE)
+	public function getIterator()
 	{
-		if ($deep) {
-			$deep = $deep > 0 ? \RecursiveIteratorIterator::SELF_FIRST : \RecursiveIteratorIterator::CHILD_FIRST;
-			return new \RecursiveIteratorIterator(new Nette\Iterators\Recursor(new \ArrayIterator($this->children)), $deep);
-
-		} else {
-			return new Nette\Iterators\Recursor(new \ArrayIterator($this->children));
-		}
+		return new \ArrayIterator($this->children);
 	}
 
 
 	/**
-	 * Returns all of children.
+	 * Returns all children.
 	 * @return array
 	 */
 	public function getChildren()
@@ -435,7 +443,7 @@ class Html extends Nette\Object implements \ArrayAccess, \Countable, \IteratorAg
 
 	/**
 	 * Renders element's start tag, content and end tag.
-	 * @param  int indent
+	 * @param  int
 	 * @return string
 	 */
 	public function render($indent = NULL)
@@ -468,7 +476,12 @@ class Html extends Nette\Object implements \ArrayAccess, \Countable, \IteratorAg
 
 	public function __toString()
 	{
-		return $this->render();
+		try {
+			return $this->render();
+		} catch (\Throwable $e) {
+		} catch (\Exception $e) {
+		}
+		trigger_error("Exception in " . __METHOD__ . "(): {$e->getMessage()} in {$e->getFile()}:{$e->getLine()}", E_USER_ERROR);
 	}
 
 
@@ -509,7 +522,15 @@ class Html extends Nette\Object implements \ArrayAccess, \Countable, \IteratorAg
 		}
 
 		$s = '';
-		foreach ($this->attrs as $key => $value) {
+		$attrs = $this->attrs;
+		if (isset($attrs['data']) && is_array($attrs['data'])) { // deprecated
+			foreach ($attrs['data'] as $key => $value) {
+				$attrs['data-' . $key] = $value;
+			}
+			unset($attrs['data']);
+		}
+
+		foreach ($attrs as $key => $value) {
 			if ($value === NULL || $value === FALSE) {
 				continue;
 
@@ -522,39 +543,38 @@ class Html extends Nette\Object implements \ArrayAccess, \Countable, \IteratorAg
 				continue;
 
 			} elseif (is_array($value)) {
-				if ($key === 'data') {
+				if (strncmp($key, 'data-', 5) === 0) {
+					$value = Json::encode($value);
+
+				} else {
+					$tmp = NULL;
 					foreach ($value as $k => $v) {
-						if ($v !== NULL && $v !== FALSE) {
-							$q = strpos($v, '"') === FALSE ? '"' : "'";
-							$s .= ' data-' . $k . '='
-								. $q . str_replace(array('&', $q), array('&amp;', $q === '"' ? '&quot;' : '&#39;'), $v)
-								. (strpos($v, '`') !== FALSE && strpbrk($v, ' <>"\'') === FALSE ? ' ' : '')
-								. $q;
+						if ($v != NULL) { // intentionally ==, skip NULLs & empty string
+							//  composite 'style' vs. 'others'
+							$tmp[] = $v === TRUE ? $k : (is_string($k) ? $k . ':' . $v : $v);
 						}
 					}
-					continue;
-				}
-
-				$tmp = NULL;
-				foreach ($value as $k => $v) {
-					if ($v != NULL) { // intentionally ==, skip NULLs & empty string
-						//  composite 'style' vs. 'others'
-						$tmp[] = $v === TRUE ? $k : (is_string($k) ? $k . ':' . $v : $v);
+					if ($tmp === NULL) {
+						continue;
 					}
-				}
-				if ($tmp === NULL) {
-					continue;
+
+					$value = implode($key === 'style' || !strncmp($key, 'on', 2) ? ';' : ' ', $tmp);
 				}
 
-				$value = implode($key === 'style' || !strncmp($key, 'on', 2) ? ';' : ' ', $tmp);
+			} elseif (is_float($value)) {
+				$value = rtrim(rtrim(number_format($value, 10, '.', ''), '0'), '.');
 
 			} else {
 				$value = (string) $value;
 			}
 
 			$q = strpos($value, '"') === FALSE ? '"' : "'";
-			$s .= ' ' . $key . '='
-				. $q . str_replace(array('&', $q), array('&amp;', $q === '"' ? '&quot;' : '&#39;'), $value)
+			$s .= ' ' . $key . '=' . $q
+				. str_replace(
+					array('&', $q, '<'),
+					array('&amp;', $q === '"' ? '&quot;' : '&#39;', self::$xhtml ? '&lt;' : '<'),
+					$value
+				)
 				. (strpos($value, '`') !== FALSE && strpbrk($value, ' <>"\'') === FALSE ? ' ' : '')
 				. $q;
 		}
